/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.core;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLDecoder;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.Set;
import java.util.Stack;
import java.util.Vector;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JOptionPane;

import ucot.Messages;
import ucot.ModuleProperties;
import ucot.exceptions.BadPropertyValueException;
import ucot.exceptions.FileFormatNotSupportedException;
import ucot.exceptions.UseCaseException;
import ucot.heuristic.AbbottsHeuristic;
import ucot.heuristic.DummyHeuristic;
import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.InputInterface;
import ucot.input.ProcessMLInputAdapter;
import ucot.input.SimpleInputAdapter;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.model.object.ObjectAnalyzeModel;
import ucot.output.DummyOutput;
import ucot.output.GXLOutput;
import ucot.output.OutputInterface;
import ucot.parser.DummyParser;
import ucot.parser.ParsedData;
import ucot.parser.ParserInterface;
import ucot.parser.SimpleParser;
import ucot.ui.DummyUI;
import ucot.ui.ProgressBarInterface;
import ucot.ui.UIInterface;
import ucot.ui.gui.GUIUtils;
import ucot.ui.gui.GraphicalUI;
import ucot.utils.PropertiesTools;
import ucot.utils.StringTools;

/**
 * <p>
 * This is the implementation of the core of the UCOT
 * program, the controller unit. The core administrates the
 * whole program by handling the analyze model and keeping
 * track and controlling the usage of all possible modules
 * loaded to the program.
 * </p>
 * 
 * <p>
 * The core component offers a <code>ControlInterface</code>
 * for the other components to request different kinds of
 * operations from the core component.
 * </p>
 * 
 * @author UCOT
 */
public class Core extends ModuleProperties implements
        ControlInterface {

    private static final String FILEFORMAT_NOT_SUPPORTED_ERROR = Messages
            .getString("FILEFORMAT_NOT_SUPPORTED");

    private static final String CANNOT_LOAD_FILES_ERROR = Messages
            .getString("CANNOT_LOAD_FILES");

    private static final String CANNOT_ADD_SELECTED_USE_CASE_ERROR = Messages
            .getString("Core.CANNOT_ADD_SELECTED_USE_CASE_ERROR");

    private static final String FILE_NOT_SAVED_ERROR = Messages
            .getString("Core.FILE_NOT_SAVED_ERROR");

    private static final String FILE_NOT_WRITABLE_ERROR = Messages
            .getString("FILE_NOT_WRITABLE");

    private static final String PLUGIN_NOT_COMPATIBLE_ERROR = Messages
            .getString("Core.PLUGIN_NOT_COMPATIBLE_ERROR");

    private static final String FILE_ALREADY_LOADED = Messages
            .getString("Core.FILE_ALREADY_LOADED_QUESTION");

    private String rootDir = "";

    /**
     * <p>
     * Returns the localized 'file not found' message.
     * </p>
     * 
     * @param file
     *            URL to the file that is not found (to be
     *            part of the message).
     * @return Localized text as a String saying <kbd>File
     *         *given file* not found!</kbd>.
     */
    public static String getFileNotFoundMessage(URL file) {
        return Messages.getFormatedMessage(
                "FILE_NOT_FOUND", file.getFile());
    }

    private InputCollection inputs = new InputCollection();

    private UseCaseCollection useCases = new UseCaseCollection();

    private AnalyzeModel analyzeModel = new ObjectAnalyzeModel();

    private Vector<HeuristicInterface> heuristics = new Vector<HeuristicInterface>();

    private Vector<ParserInterface> parsers = new Vector<ParserInterface>();

    private Vector<OutputInterface> outputs = new Vector<OutputInterface>();

    private UIInterface userInterface = new DummyUI();

    private ParserInterface currentParser = new DummyParser();

    private HeuristicInterface currentHeuristic = new DummyHeuristic();

    private ProgressBarInterface progressBar = new DummyProgressBar();

    private Logger logger = Logger.getLogger("ucot.core");

    /**
     * <p>
     * This class is a simple container for single output
     * jobs. An instance of <code>OutputJob</code> is used
     * to store the necessary data for one output operation.
     * 
     * <p>
     * Necessary data equals to the
     * <code>OutputInterface</code> of the output adapter
     * to be used for output, <code>URL</code> to the
     * target file and <code>AnalyzeModel</code> that is
     * being saved.
     * </p>
     * 
     * <p>
     * This class is required to be able to stack these
     * operations for the separate output thread running on
     * the background.
     * </p>
     * 
     * @author tujupien
     */
    private class OutputJob {
        private OutputInterface output = new DummyOutput();

        private URL url = null;

        private AnalyzeModel analyzeModel = new ObjectAnalyzeModel();
    }

    private static boolean RUNNING = true;

    private static Stack<OutputJob> OUTPUT_JOB_STACK = new Stack<OutputJob>();

    private static Thread EXECUTING_OUTPUT_THREAD = null;

    protected static Vector<Runnable> parsingThreads = new Vector<Runnable>();

    /**
     * <p>
     * Default constructor for UCOT core component. This
     * initializes the core component by loading all
     * available modules.
     * </p>
     */
    public Core() {
        // Figure out the location of the Core.class file on
        // the system.
        setRootDir();

        // Set up simple logging system.
        ConsoleHandler consoleHandler = new ConsoleHandler();
        consoleHandler.setLevel(Level.ALL);
        Logger.getLogger("ucot").addHandler(consoleHandler);
        Logger.getLogger("ucot").setLevel(Level.ALL);
        Logger.getLogger("ucot")
                .setUseParentHandlers(false);

        analyzeModel.addObserver(new AnalyzeModelLogger());

        // Inform user that program is starting.
        System.out.println(Messages
                .getString("Core.LAUNCHING"));

        // Load the input adapters that are included by
        // default in UCOT program.
        inputs.add(new SimpleInputAdapter());
        inputs.add(new ProcessMLInputAdapter());

        // Load at least GXL output adapter.
        outputs.add(new GXLOutput());

        // Load Abbott's heuristic by default.
        heuristics.add(new AbbottsHeuristic());

        // Initialize PluginLoader and load all available
        // (parser) plugins.
        PluginLoader pluginLoader = new PluginLoader(this);

        Collection<Class> parserClasses = pluginLoader
                .getClasses(ParserInterface.class);
        for (Class parserClass : parserClasses) {
            ParserInterface parser;
            try {
                Class[] args = new Class[1];
                args[0] = String.class;
                parser = (ParserInterface) parserClass
                        .getConstructor(args)
                        .newInstance(
                                new Object[] { new File(
                                        pluginLoader
                                                .getPluginDir(),
                                        pluginLoader
                                                .getDirNameForPlugin(parserClass))
                                        .toString() });
                parsers.add(parser);
            } catch (IllegalArgumentException e) {
                logger.warning(Messages.formatMessage(
                        PLUGIN_NOT_COMPATIBLE_ERROR,
                        parserClass.getCanonicalName()));
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            } catch (SecurityException e) {
                logger.warning(Messages.formatMessage(
                        PLUGIN_NOT_COMPATIBLE_ERROR,
                        parserClass.getCanonicalName()));
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            } catch (InstantiationException e) {
                logger.warning(Messages.formatMessage(
                        PLUGIN_NOT_COMPATIBLE_ERROR,
                        parserClass.getCanonicalName()));
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            } catch (IllegalAccessException e) {
                logger.warning(Messages.formatMessage(
                        PLUGIN_NOT_COMPATIBLE_ERROR,
                        parserClass.getCanonicalName()));
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            } catch (InvocationTargetException e) {
                logger.warning(Messages.formatMessage(
                        PLUGIN_NOT_COMPATIBLE_ERROR,
                        parserClass.getCanonicalName()));
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            } catch (NoSuchMethodException e) {
                logger.warning(Messages.formatMessage(
                        PLUGIN_NOT_COMPATIBLE_ERROR,
                        parserClass.getCanonicalName()));
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            }
        }

        // Load at least SimpleParser.
        parsers.add(new SimpleParser());

        // Use GraphicalUI by default.
        userInterface = new GraphicalUI(this);

        // Now load properties and apply them, also for all
        // loaded modules.
        try {
            loadProperties();
        } catch (IOException e) {
            logger.finest(StringTools
                    .exceptionStackTrace(e));
        }

        try {
            applyProperties();
        } catch (BadPropertyValueException e) {
            logger
                    .severe(Messages
                            .getFormatedMessage(
                                    "Core.CONFIGURATION_FILE_CORRUPTED_ERROR",
                                    PropertiesTools.propertiesURL));
        }
        logger.info(Messages.getString("Core.INITIALIZED"));
    }

    /**
     * <p>
     * Main method that starts the UCOT core.
     * </p>
     * 
     * @param args
     *            Command line arguments.
     */
    public static void main(String args[]) {
        new Core();
    }

    /**
     * <p>
     * Parses the current location of the Core.class file
     * and updates the class variable to match it.
     * </p>
     */
    @SuppressWarnings(
    // Suppresses type safety checking.
    value = { "deprecation" })
    private void setRootDir() {
        String rootDir = this.getClass().getResource(
                "Core.class").getFile().toString();

        // If core is run from inside a JAR package, then
        // the pwd looks
        // different
        // and should be parsed in a different way.
        rootDir = rootDir.substring(0, rootDir
                .indexOf("/ucot/core/Core.class"));
        if (rootDir.startsWith("file:")) {

            // rootDir is in URL form so create an url and
            // get the path
            // and return it in uncoded fashion so that for
            // example
            // spaces are showing in correct form.
            try {
                URL url = new URL(rootDir);
                // Uses deprecated method, but because we
                // use the only
                // internally and locally it should not
                // matter.
                rootDir = URLDecoder.decode(url.getPath());
            } catch (MalformedURLException e) {
                // Some porblems with the url.
                e.printStackTrace();

                // Failing back to old and dirty way.
                rootDir = rootDir.substring(5, rootDir
                        .length() - 1);
            }

            // Now the jar filename must be stripped from
            // the path.
            rootDir = rootDir.substring(0, rootDir
                    .lastIndexOf('/'));
        } else {
            // The real UCOT root directory is one level
            // down now when we
            // figured out that this instance is running
            // from real sources.
            // Class files are stored in 'out' (or similiar)
            // directory under the
            // root.
            rootDir += "/..";
        }

        this.rootDir = rootDir;
    }

    // ------------ CONTROL INTERFACE -------------

    /*
     * @see ucot.core.ControlInterface#getRootDir()
     */
    public String getRootDir() {
        return rootDir;
    }

    /**
     * @see ucot.core.ControlInterface#shutdown()
     */
    public void shutdown() {
        try {
            saveProperties();
        } catch (IOException e) {
            userInterface.printError(Messages
                    .formatMessage(FILE_NOT_WRITABLE_ERROR,
                            PropertiesTools.propertiesURL));
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        // Get rid of use cases, inputs, and such.
        inputs = null;
        useCases = null;
        analyzeModel = null;
        heuristics = null;
        parsers = null;
        outputs = null;
        currentParser = null;
        currentHeuristic = null;

        // TODO Remember to comment out the following line
        // when
        // running test units. Forgetting to do so will end
        // the test
        // sequence when testing for the core components
        // shutdown routine.
        System.exit(0);
    }

    /**
     * @see ucot.core.ControlInterface#loadUseCases(java.net.URL)
     */
    public void loadUseCases(URL url) {
        // See if file has already been loaded into system.
        // If it does, ask from user whether he wants to
        // refresh it.
        List<URL> urls = this.useCases.getURLs();
        if (urls.contains(url)) {
            int returnValue = GUIUtils.questionDialog(null,
                    Messages.getFormatedMessage(
                            FILE_ALREADY_LOADED, url
                                    .getFile()));

            // If user wanted to refresh file remove use
            // cases
            // from that file and continue executing method.
            // Else we just leave from method.
            if (returnValue == JOptionPane.YES_OPTION) {
                this.useCases.removeFromUrl(url);
            } else {
                return;
            }
        }

        try {
            InputInterface input = inputs
                    .getInputForUrl(url);
            useCases.merge(input.read(url));
            userInterface.useCasesLoaded();
        } catch (UseCaseException e) {
            userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.finest(StringTools
                    .exceptionStackTrace(e));
        } catch (FileFormatNotSupportedException e) {
            userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.finest(StringTools
                    .exceptionStackTrace(e));
        } catch (FileNotFoundException e) {
            userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.finest(StringTools
                    .exceptionStackTrace(e));
        } catch (Exception e) {
            userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }
    }

    /**
     * @see ucot.core.ControlInterface#loadAnalyzeModel(java.net.URL)
     */
    public void loadAnalyzeModel(URL url)
            throws IOException {
        if (url == null) return;

        // Make sure the file exists.
        File file = new File(url.getFile());
        if (!file.exists()) { throw new FileNotFoundException(
                Messages.formatMessage(Messages
                        .getString("FILE_NOT_FOUND"), url
                        .getFile())); }

        try {
            // Initialize FileInputStream and
            // ObjectInputStream.
            URLConnection connection = url.openConnection();
            connection.connect();
            InputStream urlInputStream = null;
            ObjectInputStream objectInputStream = null;
            try {
                urlInputStream = connection
                        .getInputStream();
                objectInputStream = new ObjectInputStream(
                        urlInputStream);
                // Load and typecast analyze model.
                Object loadedModel = objectInputStream
                        .readObject();
                if (loadedModel instanceof ObjectAnalyzeModel) {
                    analyzeModel = (ObjectAnalyzeModel) loadedModel;
                } else {
                    throw new FileFormatNotSupportedException(
                            FILEFORMAT_NOT_SUPPORTED_ERROR);
                }
            } finally {
                if (objectInputStream != null) objectInputStream
                        .close();
                if (urlInputStream != null) urlInputStream
                        .close();
            }
        } catch (FileFormatNotSupportedException e) {
            throw e;
        } catch (Exception e) {
            userInterface.printError(Messages
                    .formatMessage(CANNOT_LOAD_FILES_ERROR,
                            url.getFile()));
            logger.warning(e.getMessage());
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        userInterface.analyzeModelLoaded();
    }

    /**
     * @see ucot.core.ControlInterface#getUseCaseCollection()
     */
    public UseCaseCollection getUseCaseCollection() {
        return useCases;
    }

    /**
     * @see ucot.core.ControlInterface#clearAnalyzeModel()
     */
    public void clearAnalyzeModel() {
        analyzeModel.getEditor().clearModel();
        useCases.markAllUnanalyzed();
    }

    /**
     * <p>
     * This method runs the parser and heuristic in a single
     * separate thread. The thread waits first for other
     * parsing and analyzation threads invoked earlier to
     * finish and then starts the real execution.
     * </p>
     * 
     * @param runnable
     *            Runnable where this thread is running.
     * @param useCases
     *            Use cases to be parsed.
     * @param parser
     *            Parser to use.
     * @param heuristic
     *            Heuristic to use.
     */
    private void runParserAndHeuristic(Runnable runnable,
            Vector<UseCase> useCases,
            ParserInterface parser,
            HeuristicInterface heuristic) {

        // Wait for own turn to execute (older threads
        // should execute first).
        while (parsingThreads.firstElement() != runnable) {
            try {
                synchronized (this) {
                    wait();
                }
            } catch (InterruptedException e) {
            } catch (Exception e) {
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            }
        }

        // Initialize progress bar.
        progressBar.setValue(0);
        progressBar.setMaximum(useCases.size() * 100);
        progressBar.setString(Messages
                .getString("ProgressBar.INITIALIZING"));
        progressBar.setVisible(true);

        AnalyzeModel temp = null;
        int addedEntities = 0;
        Set<String> foundEntities = new HashSet<String>();
        for (UseCase useCase : useCases) {

            temp = new ObjectAnalyzeModel();

            try {
                progressBar
                        .setValue(progressBar.getValue() + 5);
                progressBar.setString(Messages
                        .getString("ProgressBar.PARSING"));

                ParsedData data = parser.parse(useCase);
                progressBar
                        .setValue(progressBar.getValue() + 55);
                progressBar
                        .setString(Messages
                                .getString("ProgressBar.RUNNING_HEURISTIC"));

                heuristic.doHeuristic(data, temp);

                progressBar
                        .setValue(progressBar.getValue() + 25);
                progressBar.setString(Messages
                        .getString("ProgressBar.MERGING"));

                useCase.setAnalyzeModel(temp);

                // Keep track of the names of the founded
                // entities.
                foundEntities.addAll(temp.getEditor()
                        .getEntityNames());

                int entitiesBefore = analyzeModel
                        .getEditor().getEntityNames()
                        .size();
                analyzeModel.getEditor().merge(temp);
                int entitiesAfter = analyzeModel
                        .getEditor().getEntityNames()
                        .size();

                // Count the difference and add addition
                // counter.
                addedEntities += (entitiesAfter - entitiesBefore);

                progressBar
                        .setValue(progressBar.getValue() + 15);
                progressBar.setString(Messages
                        .getString("ProgressBar.DONE"));

                useCase.setUseCaseAnalyzed();
            } catch (Exception e) {
                userInterface
                        .printError(CANNOT_ADD_SELECTED_USE_CASE_ERROR);
                logger.warning(StringTools
                        .exceptionStackTrace(e));
                return; // Bail out.
            } finally {
                // Hide progress bar.
                progressBar.setVisible(false);
                progressBar.setValue(0);

                // Remove this parsing thread from execution
                // list and notify
                // others.
                synchronized (parsingThreads) {
                    try {
                        parsingThreads
                                .remove(parsingThreads
                                        .firstElement());
                    } catch (NoSuchElementException e) {
                    }
                }
                synchronized (this) {
                    notifyAll();
                }
            }
        }
        userInterface.useCaseAdded(foundEntities.size(),
                addedEntities);

    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.core.ControlInterface#addToAnalyzeModel(java.util.Vector,
     *      ucot.parser.ParserInterface,
     *      ucot.heuristic.HeuristicInterface)
     */
    public void addToAnalyzeModel(
            final Vector<UseCase> useCases,
            final ParserInterface parser,
            final HeuristicInterface heuristic) {
        if (heuristic == null || parser == null) return;
        if (parsers.indexOf(parser) < 0) return;
        if (heuristics.indexOf(heuristic) < 0) return;

        // Initialize progress bar.
        progressBar = userInterface.getProgressBar();

        // Start a new parsing thread.
        synchronized (parsingThreads) {
            final Runnable runnable = new Runnable() {
                public void run() {
                    parsingThreads.add(this);
                    runParserAndHeuristic(this, useCases,
                            parser, heuristic);
                }
            };
            final Thread thread = new Thread(runnable);
            thread.start();
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.core.ControlInterface#addToAnalyzeModel(java.util.Vector)
     */
    public void addToAnalyzeModel(Vector<UseCase> useCases) {
        addToAnalyzeModel(useCases, currentParser,
                currentHeuristic);
    }

    /**
     * @see ucot.core.ControlInterface#getAnalyzeModel()
     */
    public AnalyzeModel getAnalyzeModel() {
        return analyzeModel;
    }

    /**
     * @see ucot.core.ControlInterface#getParsers()
     */
    public Vector<ParserInterface> getParsers() {
        return parsers;
    }

    /**
     * @see ucot.core.ControlInterface#getHeuristics()
     */
    public Vector<HeuristicInterface> getHeuristics() {
        return heuristics;
    }

    /**
     * @see ucot.core.ControlInterface#getOutputs()
     */
    public Vector<OutputInterface> getOutputs() {
        return outputs;
    }

    /**
     * @see ucot.core.ControlInterface#getInputs()
     */
    public InputCollection getInputs() {
        return inputs;
    }

    /**
     * @see ucot.core.ControlInterface#setCurrentParser(ucot.parser.ParserInterface)
     */
    public void setCurrentParser(ParserInterface parser) {
        currentParser = parser;
    }

    /**
     * @see ucot.core.ControlInterface#setCurrentHeuristic(ucot.heuristic.HeuristicInterface)
     */
    public void setCurrentHeuristic(
            HeuristicInterface heuristic) {
        currentHeuristic = heuristic;
    }

    /**
     * @see ucot.core.ControlInterface#getCurrentParser()
     */
    public ParserInterface getCurrentParser() {
        return currentParser;
    }

    /**
     * @see ucot.core.ControlInterface#getCurrentHeuristic()
     */
    public HeuristicInterface getCurrentHeuristic() {
        return currentHeuristic;
    }

    /**
     * @see java.lang.Object#finalize()
     */
    protected void finalize() throws Throwable {
        RUNNING = false;
        synchronized (EXECUTING_OUTPUT_THREAD) {
            if (EXECUTING_OUTPUT_THREAD != null) {
                EXECUTING_OUTPUT_THREAD.notify();
            }
        }
        super.finalize();
    }

    /**
     * <p>
     * This method executes the output operations in a
     * separate thread. This thread keeps on running once
     * started and ends execution when core's finalize
     * method has been called.
     * </p>
     */
    private void executeOutput() {
        while (RUNNING) {
            if (!OUTPUT_JOB_STACK.isEmpty()) {

                OutputJob job = null;

                synchronized (OUTPUT_JOB_STACK) {
                    job = OUTPUT_JOB_STACK.pop();
                }

                if (job == null) continue;

                try {
                    job.output.output(job.analyzeModel,
                            job.url);
                } catch (IOException e) {
                    userInterface
                            .printError(FILE_NOT_SAVED_ERROR);
                    logger.warning(StringTools
                            .exceptionStackTrace(e));
                }

                userInterface.exportDone();
            } else {
                try {
                    synchronized (EXECUTING_OUTPUT_THREAD) {
                        EXECUTING_OUTPUT_THREAD.wait();
                    }
                } catch (Exception e) {
                    logger.warning(StringTools
                            .exceptionStackTrace(e));
                }
            }
        }
    }

    /**
     * @see ucot.core.ControlInterface#output(java.net.URL,
     *      ucot.output.OutputInterface,
     *      ucot.model.AnalyzeModel)
     */
    public void output(URL url, OutputInterface output,
            AnalyzeModel model) throws Exception {
        OutputJob job = new OutputJob();
        job.url = url;
        job.output = (output != null ? output
                : new DummyOutput());
        job.analyzeModel.getEditor().merge(model);

        synchronized (OUTPUT_JOB_STACK) {
            OUTPUT_JOB_STACK.push(job);
        }
        if (EXECUTING_OUTPUT_THREAD == null) {
            EXECUTING_OUTPUT_THREAD = new Thread(
                    new Runnable() {
                        public void run() {
                            executeOutput();
                        }
                    });
            EXECUTING_OUTPUT_THREAD.start();
        } else {
            synchronized (EXECUTING_OUTPUT_THREAD) {
                EXECUTING_OUTPUT_THREAD.notify();
            }
        }
    }

    /**
     * @see ucot.core.ControlInterface#saveAnalyzeModel(java.net.URL)
     */
    public void saveAnalyzeModel(URL url) {
        if (url == null) return;

        try {
            if (url.getProtocol().equalsIgnoreCase("file")) {
                File file = new File(url.getFile());
                if (!file.canWrite() && file.exists()) {
                    userInterface
                            .printError(Messages
                                    .formatMessage(
                                            FILE_NOT_WRITABLE_ERROR,
                                            url.getFile()));
                    return;
                }

                FileOutputStream fileOutputStream = null;
                ObjectOutputStream objectOutputStream = null;

                try {
                    fileOutputStream = new FileOutputStream(
                            file);
                    objectOutputStream = new ObjectOutputStream(
                            fileOutputStream);
                    // Save model.
                    objectOutputStream
                            .writeObject((ObjectAnalyzeModel) analyzeModel);
                } finally {
                    if (objectOutputStream != null) objectOutputStream
                            .close();
                    if (fileOutputStream != null) fileOutputStream
                            .close();
                }
            }
        } catch (Exception e) {
            userInterface.printError(FILE_NOT_SAVED_ERROR);
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }
    }

    /**
     * @see ucot.core.ControlInterface#reloadUseCases(java.net.URL)
     */
    public void reloadUseCases(URL url) {
        useCases.removeFromUrl(url);
        loadUseCases(url);
    }

    /**
     * @see ucot.core.ControlInterface#applyProperties()
     */
    public void applyProperties()
            throws BadPropertyValueException {
        Vector<BadPropertyValueException> exceptions = new Vector<BadPropertyValueException>();
        // Find and set the current parser.
        currentParser = findParser(properties
                .getProperty(PROPERTY_CURRENT_PARSER));

        // If the specified one wasn't found.
        if (currentParser == null && parsers.size() > 0) {
            // Select the first one from the list...
            currentParser = parsers.get(0);
            // and update properties.
            properties.setProperty(PROPERTY_CURRENT_PARSER,
                    currentParser.getName());
            exceptions
                    .add(new BadPropertyValueException(
                            Messages
                                    .formatMessage(
                                            Messages
                                                    .getString("Core.PARSER_NOT_FOUND"),
                                            properties
                                                    .getProperty(PROPERTY_CURRENT_PARSER))));
        }

        // Find and set current heuristic.
        currentHeuristic = findHeuristic(properties
                .getProperty(PROPERTY_CURRENT_HEURISTIC));

        // If the specified one wasn't found.
        if (currentHeuristic == null
                && heuristics.size() > 0) {
            // Select the first one from the list...
            currentHeuristic = heuristics.get(0);
            // and update properties.
            properties.setProperty(
                    PROPERTY_CURRENT_HEURISTIC,
                    currentHeuristic.getName());
            exceptions
                    .add(new BadPropertyValueException(
                            Messages
                                    .formatMessage(
                                            Messages
                                                    .getString("Core.HEURISTIC_NOT_FOUND"),
                                            properties
                                                    .getProperty(PROPERTY_CURRENT_HEURISTIC))));
        }

        // Make input adapters apply their properties.
        for (int i = 0; i < inputs.getInputCount(); i++) {
            try {
                inputs.getInput(i).applyProperties();
            } catch (BadPropertyValueException e) {
                exceptions.add(e);
            }
        }

        // Make heuristic adapters to apply their
        // properties.
        for (HeuristicInterface heuristic : heuristics) {
            try {
                heuristic.applyProperties();
            } catch (BadPropertyValueException e) {
                exceptions.add(e);
            }
        }

        // Make parser adapters to apply their properties.
        for (ParserInterface parser : parsers) {
            try {
                parser.applyProperties();
            } catch (BadPropertyValueException e) {
                exceptions.add(e);
            }
        }

        // Make output adapters to apply their properties.
        for (OutputInterface output : outputs) {
            try {
                output.applyProperties();
            } catch (BadPropertyValueException e) {
                exceptions.add(e);
            }
        }

        // Now apply user interface's properties.
        try {
            userInterface.applyProperties();
        } catch (BadPropertyValueException e) {
            exceptions.add(e);
        }

        String exceptionMessage = "";
        for (BadPropertyValueException e : exceptions) {
            exceptionMessage += "\n" + e.getMessage();
        }
        if (!exceptions.isEmpty()) {
            exceptionMessage = exceptionMessage
                    .substring(1);
            throw new BadPropertyValueException(
                    exceptionMessage);
        }
    }

    /**
     * <p>
     * Helper method for applyProperties to find parsers by
     * name.
     * </p>
     * 
     * @param name
     *            Parser to search.
     * @return Found parser or null if no parser with given
     *         name existed.
     */
    private ParserInterface findParser(String name) {
        for (int i = 0; i < parsers.size(); i++) {
            if (parsers.get(i).getName().equals(name)) { return parsers
                    .get(i); }
        }
        return null;
    }

    /**
     * Helper method for applyProperties to find heuristics
     * by name.
     * 
     * @param name
     *            Heuristic to search.
     * @return Found heuristic or null if no heuristic with
     *         given name existed.
     */
    private HeuristicInterface findHeuristic(String name) {
        for (int i = 0; i < heuristics.size(); i++) {
            if (heuristics.get(i).getName().equals(name)) { return heuristics
                    .get(i); }
        }
        return null;
    }

    /**
     * @see ucot.ModulePropertyInterface#loadDefaultProperties()
     */
    public Properties loadDefaultProperties() {
        Properties result = super.loadDefaultProperties();
        // TODO Set default properties here, for example:
        // FIXME: XXXXXX What if Stanford parser adapter is
        // not loaded??
        result.setProperty(PROPERTY_CURRENT_PARSER,
                "Stanford parser");
        result.setProperty(PROPERTY_CURRENT_HEURISTIC,
                "Abbott's heuristic");
        return result;
    }

    /**
     * @see ucot.ModulePropertyInterface#saveProperties()
     */
    public synchronized void saveProperties()
            throws IOException {
        Vector<IOException> exceptions = new Vector<IOException>();
        try {
            super.saveProperties();
        } catch (IOException e) {
            exceptions.add(e);
        }

        // Make input adapters save their properties.
        for (int i = 0; i < inputs.getInputCount(); i++) {
            try {
                inputs.getInput(i).saveProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Make heuristic adapters to save their properties.
        for (HeuristicInterface heuristic : heuristics) {
            try {
                heuristic.saveProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Make parser adapters to save their properties.
        for (ParserInterface parser : parsers) {
            try {
                parser.saveProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Make output adapters to save their properties.
        for (OutputInterface output : outputs) {
            try {
                output.saveProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Now save user interface's properties.
        try {
            userInterface.saveProperties();
        } catch (IOException e) {
            exceptions.add(e);
        }

        String exceptionMessage = "";
        for (IOException e : exceptions) {
            exceptionMessage += "\n" + e.getMessage();
        }
        if (!exceptions.isEmpty()) {
            exceptionMessage = exceptionMessage
                    .substring(1);
            throw new IOException(exceptionMessage);
        }
    }

    /**
     * @see ucot.ModulePropertyInterface#loadProperties()
     */
    public synchronized void loadProperties()
            throws IOException {
        Vector<IOException> exceptions = new Vector<IOException>();
        try {
            super.loadProperties();
        } catch (IOException e) {
            exceptions.add(e);
        }

        // Make input adapters load their properties.
        for (int i = 0; i < inputs.getInputCount(); i++) {
            try {
                inputs.getInput(i).loadProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Make heuristic adapters to load their properties.
        for (HeuristicInterface heuristic : heuristics) {
            try {
                heuristic.loadProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Make parser adapters to load their properties.
        for (ParserInterface parser : parsers) {
            try {
                parser.loadProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Make output adapters to load their properties.
        for (OutputInterface output : outputs) {
            try {
                output.loadProperties();
            } catch (IOException e) {
                exceptions.add(e);
            }
        }

        // Now load user interface's properties.
        try {
            userInterface.loadProperties();
        } catch (IOException e) {
            exceptions.add(e);
        }

        String exceptionMessage = "";
        for (IOException e : exceptions) {
            exceptionMessage += "\n" + e.getMessage();
        }
        if (!exceptions.isEmpty()) {
            exceptionMessage = exceptionMessage
                    .substring(1);
            throw new IOException(exceptionMessage);
        }
    }
}
