/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.core;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * <p>
 * This is class loader for UCOT-programs plugins. It is
 * used to load plugins from spesific directory. Currently
 * it does not handle jar files properly so only plain class
 * files can be loaded.
 * </p>
 * <p>
 * This class loader delegates class loading normally to its
 * parents. If the classes are not found on general
 * classpaths then classes are looked under the directory
 * set at the construction time. Classes are located normal
 * way: packages are directories and classes are files
 * ending with <code>.class</code>-extension.
 * </p>
 * 
 * @see ucot.core.PluginLoader
 */

public class PluginClassLoader extends ClassLoader {
    File pluginDir;

    /**
     * This is file filter for listin jar files.
     */
    @SuppressWarnings("unused")
    private FileFilter jarFilter = new FileFilter() {
        public boolean accept(File pathname) {
            return pathname.isFile()
                    && pathname.getAbsolutePath().endsWith(
                            "jar");
        }
    };

    /**
     * Costructs the classloader for spesific directory.
     * 
     * @param dir
     *            The directory for the class loader.
     */
    public PluginClassLoader(File dir) {
        pluginDir = dir;

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.ClassLoader#findClass(java.lang.String)
     */
    @Override
    protected Class<?> findClass(String name)
            throws ClassNotFoundException {
        byte[] b = loadClassData(name);
        return defineClass(name, b, 0, b.length);
    }

    /**
     * Loads class data under the directory defined for this
     * classloader. It modifies the class name so that
     * <code>package.package1.ClassABCD</code> means file
     * <code>package/package1/ClassABCD.class</code>.
     * Then the file is located and if its is not found then
     * <code>ClassNotFoundException</code> is thrown.
     * 
     * @param name
     *            The class name we need to load.
     * @return The data for the class.
     * @throws ClassNotFoundException
     *             Thrown if file for the class is not
     *             found.
     */
    protected byte[] loadClassData(String name)
            throws ClassNotFoundException {
        String fileName = name.replace('.',
                File.separatorChar)
                + ".class";
        File classFile = new File(pluginDir, fileName);
        if (classFile.exists()) {
            assert classFile.length() < Integer.MAX_VALUE;
            byte[] data = new byte[(int) classFile.length()];
            try {

                InputStream in = new FileInputStream(
                        classFile);
                try {
                    in.read(data);
                } finally {
                    in.close();
                }
                return data;
            } catch (IOException e) {
                e.printStackTrace();
                throw new ClassNotFoundException(
                        "Could not found class :" + name, e);
            }
        } else {
            // XXX Implement jar file loading.

            // Locate jar files.
            // Look for properly named file.
            // Load its data and return it.

            /*
             * File[] jars = pluginDir.listFiles(jarFilter);
             * for(File file : jars){ JarFile jarFile = new
             * JarFile(file); Enumeration<JarEntry> en =
             * jarFile.entries();
             * while(en.hasMoreElements()){ JarEntry entry =
             * en.nextElement();
             * System.out.println(entry.getName()); } }
             */
            throw new ClassNotFoundException();
        }
    }

}
