/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.core;

import static java.util.logging.Logger.getLogger;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;
import java.util.logging.Logger;

import org.xml.sax.helpers.ParserAdapter;

import ucot.Messages;

/**
 * <p>
 * Handles plugin loading. This class finds all the
 * subdirectories under spesific direcotyr. Those spesific
 * subdirectories are considered plugin directories. Very
 * simple configuration file is expected to be found under
 * the plugin directory. This file is called
 * <code>plugin.properties</code> and it is normal java
 * properties file.
 * </p>
 * <p>
 * Here is example of plugin.properties file that defines
 * plugin called "Adapter". Property called class defines
 * the plugins main class that is loaded under this plugin
 * loader.
 * 
 * <pre><code>
 *       name=Adapter
 *       class=package1.AdapterClass
 * </code></pre>
 * 
 * </p>
 * <p>
 * To use loaded plugins the program asks loaded class with
 * <code>getClasses()</code> or
 * <code>getClasses(Class)</code>
 * 
 * </p>
 * 
 */
// XXX JATKA TST
public class PluginLoader {

    private static final String DEFAULT_PLUGIN_DIR_NAME = "plugins";

    private static final String DEFAULT_PLUGIN_PROPERTIES_FILE = 
        "plugin.properties";

    private static final String PLUGIN_DIR_NOT_EXIST_LOGMESSAGE = 
        "PluginLoader.PLUGIN_DIR_NOT_EXIST_LOGMESSAGE";

    private static final String PLUGIN_DIR_IS_NOT_DIR_LOGMESSAGE = 
        "PluginLoader.PLUGIN_DIR_IS_NOT_DIR_LOGMESSAGE";

    private static final String LOADER_CHECKING_DIR_LOGMESSAGE = 
        "PluginLoader.LOADER_CHECKING_DIR_LOGMESSAGE";

    private static final String FOUND_MAIN_DIR_LOGMESSAGE = 
        "PluginLoader.FOUND_MAIN_DIR_LOGMESSAGE";

    private static final String NO_PLUGIN_PROPERTIES_FOUND_LOGMESSAGE = 
        "PluginLoader.NO_PLUGIN_PROPERTIES_FOUND_LOGMESSAGE";

    private static final String ERROR_READING_FILE_LOGMESSAGE = 
        "PluginLoader.ERROR_READING_FILE_LOGMESSAGE";

    private static final String PLUGIN_CLASS_NOT_DEFINED_LOGMESSAGE = 
        "PluginLoader.PLUGIN_CLASS_NOT_DEFINED_LOGMESSAGE";

    private static final String PLUGIN_NAME_NOT_DEFINED_LOGMESSAGE = 
        "PluginLoader.PLUGIN_NAME_NOT_DEFINED_LOGMESSAGE";

    private static final String LOADING_PLUGIN_LOGMESSAGE = 
        "PluginLoader.LOADING_PLUGIN_LOGMESSAGE";

    private static final String PLUGIN_CLASS_NOT_FOUND_LOGMESSAGE = 
        "PluginLoader.PLUGIN_CLASS_NOT_FOUND_LOGMESSAGE";

    private static final String LOADED_PLUGIN_LOGMESSAGE = 
        "PluginLoader.LOADED_PLUGIN_LOGMESSAGE";

    private static final String CLASS_PROPERTY_KEY = "class";

    private static final String NAME_PROPERTY_KEY = "name";

    File pluginMainDir;

    Logger logger = getLogger("ucot.core");

    Map<Class, String> pluginClasses = new HashMap<Class, String>();

    // TODO comment me
    public File getPluginDir() {
        return pluginMainDir;
    }

    // TODO comment me
    public PluginLoader(Core core) {
        pluginMainDir = new File(core.getRootDir(),
                DEFAULT_PLUGIN_DIR_NAME);
        if (!pluginMainDir.exists()) {
            logger.severe(Messages.getFormatedMessage(
                    PLUGIN_DIR_NOT_EXIST_LOGMESSAGE,
                    pluginMainDir.getAbsolutePath()));
            return;
        }
        if (!pluginMainDir.isDirectory()) {
            logger.severe(Messages.getFormatedMessage(
                    PLUGIN_DIR_IS_NOT_DIR_LOGMESSAGE,
                    pluginMainDir.getAbsolutePath()));
            ;
            return;
        }
        logger.info("Plugin loader initalized.");
        File[] pluginDirs = pluginMainDir
                .listFiles(new FileFilter() {
                    public boolean accept(File pathname) {
                        return pathname.isDirectory();
                    }
                });
        if (null != pluginDirs) {
            logger.finer(Messages.getFormatedMessage(
                    FOUND_MAIN_DIR_LOGMESSAGE,
                    pluginMainDir.getAbsolutePath()));
            for (File pluginDir : pluginDirs) {
                // load plugin.
                logger.finer(Messages.getFormatedMessage(
                        LOADER_CHECKING_DIR_LOGMESSAGE,
                        pluginDir));
                File pluginPropertiesFile = new File(
                        pluginDir,
                        DEFAULT_PLUGIN_PROPERTIES_FILE);
                if (pluginPropertiesFile.exists()) {
                    Properties pluginProperties = new Properties();
                    try {

                        InputStream in = new FileInputStream(
                                pluginPropertiesFile);
                        try {
                            pluginProperties.load(in);
                        } finally {
                            in.close();
                        }
                    } catch (IOException e) {
                        logger
                                .severe(Messages
                                        .getFormatedMessage(
                                                ERROR_READING_FILE_LOGMESSAGE,
                                                pluginPropertiesFile
                                                        .getAbsolutePath()));
                    }

                    String name = pluginProperties
                            .getProperty(NAME_PROPERTY_KEY);
                    String className = pluginProperties
                            .getProperty(CLASS_PROPERTY_KEY);
                    if (null == className) {
                        logger
                                .warning(Messages
                                        .getFormatedMessage(
                                                PLUGIN_CLASS_NOT_DEFINED_LOGMESSAGE,
                                                pluginDir));
                    }
                    if (null == name) {
                        logger
                                .warning(Messages
                                        .getFormatedMessage(
                                                PLUGIN_NAME_NOT_DEFINED_LOGMESSAGE,
                                                pluginDir));
                    }
                    if (null != className && null != name) {
                        logger
                                .fine(Messages
                                        .getFormatedMessage(
                                                LOADING_PLUGIN_LOGMESSAGE,
                                                pluginDir,
                                                name,
                                                className));
                        PluginClassLoader loader = new PluginClassLoader(
                                pluginDir);
                        try {
                            Class klass = loader
                                    .findClass(className);
                            pluginClasses.put(klass,
                                    pluginDir.getName());
                            logger
                                    .fine(Messages
                                            .getFormatedMessage(
                                                    LOADED_PLUGIN_LOGMESSAGE,
                                                    pluginDir,
                                                    name,
                                                    className));
                        } catch (ClassNotFoundException e) {
                            logger
                                    .severe(Messages
                                            .getFormatedMessage(
                                                    PLUGIN_CLASS_NOT_FOUND_LOGMESSAGE,
                                                    pluginDir,
                                                    name,
                                                    className));
                        }

                    }

                } else {
                    logger
                            .warning(Messages
                                    .getFormatedMessage(
                                            NO_PLUGIN_PROPERTIES_FOUND_LOGMESSAGE,
                                            pluginDir));
                }

            }
        }
    }

    // TODO comment me
    public Collection<Class> getClasses() {
        Vector<Class> ret = new Vector<Class>(pluginClasses
                .size());
        ret.addAll(pluginClasses.keySet());
        return ret;
    }

    // TODO comment me
    @SuppressWarnings(value = { "unchecked" } // Suppreess
    // type
    // safty
    // checking.
    )
    public Collection<Class> getClasses(Class reqApi) {
        Collection<Class> ret = getClasses();
        Iterator<Class> iterator = ret.iterator();

        while (iterator.hasNext()) {
            Class klass = iterator.next();
            try {
                boolean foundInterface = false;
                for (Class interfase : klass
                        .getInterfaces()) {
                    if (interfase.equals(reqApi)) {
                        foundInterface = true;
                        break;
                    }
                }
                if (!foundInterface) klass = klass
                        .asSubclass(ParserAdapter.class);
            } catch (ClassCastException e) {
                // Ok. This was not proper.
                iterator.remove();
            }
        }

        return ret;
    }

    // TODO comment me
    public String getDirNameForPlugin(Class klass) {
        return pluginClasses.get(klass);
    }

    /*
     * public static void main(String args[]){ // Set up
     * simple logging system. ConsoleHandler consoleHandler =
     * new ConsoleHandler();
     * consoleHandler.setLevel(Level.ALL);
     * Logger.getLogger("ucot").addHandler(consoleHandler);
     * Logger.getLogger("ucot").setLevel(Level.ALL);
     * Logger.getLogger("ucot").setUseParentHandlers(false);
     * PluginLoader pl = new PluginLoader(); }
     */
}
