/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.model;

import java.io.Serializable;

/**
 * <p>
 * Class contains updation information about the model. It
 * contains the type of change and information about the
 * change.
 * </p>
 * <p>
 * For example deletion of method would look like this:
 * </p>
 * 
 * <pre>
 * new Updatition(UpdationType.DELETION,
 *         UpdatioDatatype.METHOD, &quot;entity&quot;, &quot;method&quot;);
 * </pre>
 * 
 * <p>
 * It can be used to signal that the "method" of "entity" is
 * deleted.
 * </p>
 * 
 * @author pajumasu
 */
public class Updation implements Serializable {
    public static final long serialVersionUID = 121289714289124L;

    /**
     * Contains information about the updation type.
     * 
     * @author pajumasu
     * 
     */
    public enum UpdationType {
        /**
         * No operation.
         */
        NOP,
        /**
         * Modification occurred.
         */
        MODIFY,
        /**
         * Cration ocurred.
         */
        NEW,
        /**
         * Deletion ocurred.
         */
        DELETION,

        /**
         * Model has just been completely wiped out.
         */
        CLEAR,

        /**
         * Big chunk of modification is done.
         */
        READY,
        /**
         * Big chunk of modification started. Time intensive
         * updations should be cased and wait for ready
         * signal.
         */
        MODIFICATION_STARTED
    };

    /**
     * Contains information about the type beign changed.
     * 
     * @author pajumasu
     */
    public enum UpdationDataType {
        /**
         * Entity is beign altered. Reference contains the
         * name of the entity.
         */
        ENTITY,
        /**
         * Method is beign altered. Reference contains the
         * name of the entity and the name of the method.
         */
        METHOD,
        /**
         * Attribute is beign altered. Reference contains
         * the name of the entity and the name of the
         * attribute.
         */
        ATTRIBUTE,
        /**
         * Attribute to cardinality is beign altered.
         * Reference contains the name of the entity and the
         * name of the attribute and the to cardinality.
         */
        ATTRIBUTE_TO_CARDINALITY,
        /**
         * Attribute to cardinality is beign altered.
         * Reference contains the name of the entity and the
         * name of the attribute and the from cardinality.
         */
        ATTRIBUTE_FROM_CARDINALITY,
        /**
         * Method influence is beign altered. Reference
         * contains the name of the entity and the name of
         * the methdod and the name of the entity that is
         * influenced.
         */
        METHOD_INFLUENCE,
        /**
         * Parent relation is beign altered. Reference
         * contains the name of the entity which parent is
         * changed and the entity that is set to the first
         * ones parent.
         */
        ENTITY_PARENT,
        /**
         * The type is beign altered Reference contains the
         * name of the entity which type is changed and the
         * type.
         */
        ENTITY_TYPE,
        /**
         * No data caried.
         */
        EMPTY
    };

    UpdationType type;

    UpdationDataType dataType = UpdationDataType.EMPTY;

    String[] references = new String[0];

    /**
     * <p>
     * Initializes empty instance <code>Updation</code>.
     * <code>UpdationType</code> of this is "no operation"
     * and this does not carry any data.
     * </p>
     * 
     */
    public Updation() {
        this(UpdationType.NOP, UpdationDataType.EMPTY);
    }

    /**
     * <p>
     * Initializes new instance of <code>Updation</code>
     * which has the given <code>UpdationType</code> and
     * <code>UpdationDataType</code> as values.
     * </p>
     * 
     * @param type
     *            <code>UpdationType</code> of this
     *            <code>Updation</code>
     * @param dataType
     *            <code>UpdationDataType</code> of this
     *            <code>Updation</code>
     */
    public Updation(UpdationType type,
            UpdationDataType dataType) {
        this.type = type;
        this.dataType = dataType;
    }

    /**
     * <p>
     * Initializes new instance of <code>Updation</code>
     * which has the given <code>UpdationType</code>,
     * <code>UpdationDataType</code> and references as
     * values.
     * </p>
     * 
     * @param type
     *            <code>UpdationType</code> of this
     *            <code>Updation</code>
     * @param dataType
     *            <code>UpdationDataType</code> of this
     *            <code>Updation</code>
     * @param references
     *            References of this <code>Updation</code>.
     *            The references that should be carried
     *            depend on the used
     *            <code>UpdationDataType</code>.
     * 
     * @see UpdationDataType
     * 
     */
    public Updation(UpdationType type,
            UpdationDataType dataType, String... references) {
        this.type = type;
        this.dataType = dataType;
        this.references = references;
    }

    /**
     * <p>
     * Initializes new instance of <code>Updation</code>
     * which has the given <code>UpdationType</code> as
     * value.
     * </p>
     * 
     * @param type
     *            <code>UpdationType</code> of this
     *            <code>Updation</code>
     */
    public Updation(UpdationType type) {
        this.type = type;
    }

    /**
     * <p>
     * Returns the <code>UpdationDataType</code> of this
     * instance of <code>updation</code>.
     * </p>
     * 
     * @return <code>UpdationDataType</code> of this
     *         <code>Updation</code>.
     */
    public UpdationDataType getDataType() {
        return dataType;
    }

    /**
     * <p>
     * Returns the references carried by this instance of
     * <code>updation</code>.
     * </p>
     * <p>
     * What the references are can be deciphered by checking
     * the <code>UpdationDataType</code> of this
     * <code>Updation</code>.
     * </p>
     * 
     * @return References carried by <code>Updation</code>.
     * 
     * @see UpdationDataType
     * @see #getDataType()
     * 
     */
    public String[] getReferences() {
        return references;
    }

    /**
     * <p>
     * Returns the <code>UpdationType</code> of this
     * instance of <code>Updation</code>.
     * </p>
     * 
     * @return <code>UpdationType</code> of this
     *         <code>Updation</code>.
     */
    public UpdationType getType() {
        return type;
    }

}
