/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.model.object;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import ucot.utils.StringTools;

/**
 * <p>
 * This class represents entity in analyze model.
 * </p>
 * 
 * @author pajumasu
 */
public class Entity implements Serializable {
    public static final long serialVersionUID = 2368572046870269076L;

    /**
     * Maps attribute names and attributes. Attribute names
     * are not in use currently.
     */
    private Map<String, Attribute> attributes = new HashMap<String, Attribute>();

    /**
     * Maps method names and methods.
     */
    private Map<String, Method> methods = new HashMap<String, Method>();

    /**
     * The child entities.
     */
    private Set<Entity> childEntities = new HashSet<Entity>();

    /**
     * The parent entities.
     */
    private Set<Entity> parentEntities = new HashSet<Entity>();

    /**
     * The name of this entity.
     */
    private String name;

    /**
     * The type of the entity.
     */
    private String type;

    /**
     * Marks if this entity is deleted or not. Because there
     * are many references to entities inside the model it
     * is easier to mark entity deletet when its is removed
     * from the model and the unnessesary references are
     * removed when they are needed. This way we dont need
     * to know all the places for the references and remove
     * them on site when entity is deleted.
     */
    boolean deleted = false;

    /**
     * <p>
     * Creates the entity.
     * </p>
     * <p>
     * The name is modified so that the first letter is
     * always an uppercase letter.
     * </p>
     * 
     * @param name
     *            Name of this entity.
     */
    public Entity(String name) {
        setName(name);
    }

    /**
     * <p>
     * Sets the name of the entity.
     * </p>
     * <p>
     * The name is modified so that the first letter is
     * always an uppercase letter.
     * </p>
     * 
     * @param name
     *            Name of this entity.
     */
    protected void setName(String name) {
        this.name = StringTools.capitalize(name);
    }

    /**
     * <p>
     * Removes deleted entities from the attribute list.
     * </p>
     */
    private void updateAttributes() {
        // Rebuild whole attribute map.
        Map<String, Attribute> newAttributesMap = new HashMap<String, Attribute>();

        for (Iterator<Map.Entry<String, Attribute>> it = attributes
                .entrySet().iterator(); it.hasNext();) {
            Map.Entry<String, Attribute> entry = it.next();
            // Entity entity = entry.getValue().getEntity();
            // if (entity.deleted) it.remove();
            if (!entry.getValue().getEntity().deleted) {
                newAttributesMap.put(entry.getValue()
                        .getName(), entry.getValue());
            }
        }

        attributes = newAttributesMap;
    }

    /**
     * <p>
     * Adds attribute for this entity
     * </p>
     * 
     * @param name
     *            Name of the <code>Attribute</code>.
     * @param attribute
     *            The <code>Attribute</code> to add.
     */
    public void addAttribute(String name,
            Attribute attribute) {
        attributes.put(StringTools.capitalize(name),
                attribute);
    }

    /**
     * <p>
     * Returns attribute called name.
     * </p>
     * 
     * @param name
     *            Name of the <code>Attribute</code> to
     *            return.
     * @return The <code>Attribute</code> called 'name'.
     */
    public Attribute getAttribute(String name) {
        updateAttributes();
        return attributes.get(StringTools.capitalize(name));
    }

    /**
     * <p>
     * Returns all the attributes.
     * </p>
     * 
     * @return <code>Attributes</code> of this entity in a
     *         Set.
     */
    public Set<Attribute> getAttributes() {
        updateAttributes();
        Set<Attribute> ret = new HashSet<Attribute>();
        ret.addAll(attributes.values());
        return ret;
    }

    /**
     * <p>
     * Removes attribute called name.
     * </p>
     * 
     * @param name
     *            Name of the attribute to remove.
     */
    public void removeAttribute(String name) {
        attributes.remove(name);
    }

    /**
     * <p>
     * Removes given attribute.
     * </p>
     * 
     * @param toBeRemoved
     *            <code>Attribute</code> that should be
     *            removed.
     */
    public void removeAttribute(Attribute toBeRemoved) {
        for (Iterator<Map.Entry<String, Attribute>> it = attributes
                .entrySet().iterator(); it.hasNext();) {
            Map.Entry<String, Attribute> entry = it.next();
            Attribute attribute = entry.getValue();
            if (attribute.equals(toBeRemoved)) {
                it.remove();
            }
        }
    }

    /**
     * <p>
     * Adds attribute.
     * </p>
     * <p>
     * Attributes name is set based on the name of the
     * entity defined in attribute.
     * </p>
     * 
     * @param attribute
     *            <code>Attribute</code> to add.
     */
    public void addAttribute(Attribute attribute) {
        attributes.put(attribute.getEntity().getName(),
                attribute);
    }

    /**
     * <p>
     * Returns the name of this entity.
     * </p>
     * 
     * @return Name of this entity.
     */
    public String getName() {
        return name;
    }

    /**
     * Removes deleted entites from given collection.
     * 
     * @param col
     *            Collection of entities.
     */
    private void checkEntityCollection(
            Collection<Entity> col) {
        for (Iterator<Entity> it = col.iterator(); it
                .hasNext();) {
            Entity entity = it.next();
            if (entity.deleted) {
                it.remove();
                entity.removeChild(this);
            }
        }
    }

    /**
     * <p>
     * Returns the parent entities of this entity.
     * </p>
     * 
     * @return Parent entities of this entity.
     */
    public Set<Entity> getParents() {
        checkEntityCollection(parentEntities);
        Set<Entity> ret = new HashSet<Entity>();
        ret.addAll(parentEntities);
        return parentEntities;
    }

    /**
     * <p>
     * Returns the children of this entity.
     * </p>
     * 
     * @return Set of children of this entity.
     */
    public Set<Entity> getChildren() {
        checkEntityCollection(childEntities);
        Set<Entity> ret = new HashSet<Entity>();
        ret.addAll(childEntities);
        return ret;
    }

    /**
     * <p>
     * Adds parent for this <code>Entity</code>. Also
     * adds this entity as a child of the given entity if
     * nessesary.
     * </p>
     * 
     * @param entity
     *            <code>Entity</code> that should be added
     *            as parent.
     */
    public void addParent(Entity entity) {
        if (null == entity) return;
        if (this == entity) return;
        if (entity.deleted) return;
        if (parentEntities.add(entity)) entity
                .addChild(this);
    }

    /**
     * <p>
     * Adds child for this <code>Entity</code>. Also adds
     * this entity as a parent of the given entity if
     * nessesary
     * </p>
     * 
     * @param entity
     *            <code>Entity</code> that should be added
     *            as child.
     */
    public void addChild(Entity entity) {
        if (null == entity) return;
        if (this == entity) return;
        if (entity.deleted) return;
        if (childEntities.add(entity)) entity
                .addParent(this);
    }

    /**
     * <p>
     * Removes parent of this entity and also removes this
     * entity from the child list of a given entity if
     * nessesary.
     * </p>
     * 
     * @param entity
     *            <code>Entity</code> that should be
     *            removed from this entitys parents.
     */
    public void removeParent(Entity entity) {
        if (null == entity) return;
        if (parentEntities.remove(entity)) entity
                .removeChild(this);
    }

    /**
     * <p>
     * Removes child of this entity and also removes this
     * entity from the parent list of a given entity if
     * nessesary.
     * </p>
     * 
     * @param entity
     *            <code>Entity</code> that should be
     *            removed from this entitys children.
     */
    public void removeChild(Entity entity) {
        if (null == entity) return;
        if (childEntities.remove(entity)) entity
                .removeParent(this);
    }

    /**
     * <p>
     * Returns <code>Method</code> of this
     * <code>Entity</code> that is called 'name'.
     * </p>
     * 
     * @param methodName
     *            Name of the <code>Method</code>.
     * @return method <code>Method</code> called 'name'.
     */
    public Method getMethod(String methodName) {
        return methods.get(methodName.toLowerCase());
    }

    /**
     * <p>
     * Creates and adds <code>Method</code> for this
     * <code>Entity</code> using given name.
     * </p>
     * 
     * @param methodName
     *            Name of the <code>Method</code> to add.
     */
    public void addMethod(String methodName) {
        Method method = methods.get(methodName
                .toLowerCase());
        if (null == method) {
            method = new Method(methodName);
            methods.put(method.getName(), method);
        }

    }

    /**
     * <p>
     * Adds method for this entity.
     * </p>
     * 
     * @param method
     *            <code>Method</code> to add.
     */
    public void addMethod(Method method) {
        methods.put(method.getName(), method);
    }

    /**
     * <p>
     * Removes method called 'name'.
     * </p>
     * 
     * @param methodName
     *            Name of the method to remove.
     */
    public void removeMethod(String methodName) {
        methods.remove(methodName);
    }

    /**
     * <p>
     * Returns the names of the methods.
     * </p>
     * 
     * @return Set of method names of this
     *         <code>Entity</code>.
     */
    public Set<String> getMethodNames() {
        return methods.keySet();
    }

    /**
     * <p>
     * Return the type of this entity.
     * </p>
     * 
     * @return The type of this entity.
     */
    public String getType() {
        return type;
    }

    /**
     * <p>
     * Sets the type for this entity.
     * </p>
     * 
     * @param type
     *            The type
     */
    public void setType(String type) {
        this.type = type;
    }

    /**
     * <p>
     * Get all the methods of this <code>Entity</code>.
     * </p>
     * 
     * @return Set of <code>Methods</code> of this
     *         <code>Entity</code>.
     */
    public Set<Method> getMethods() {
        Set<Method> ret = new HashSet<Method>();
        ret.addAll(methods.values());
        return ret;
    }
}
