/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.parser;

import java.util.List;
import java.util.Vector;

/**
 * <p>
 * "Word" of a sentence. It does not nessesary contain a
 * single word but can also contain for example an noun
 * phrase (blak car). <code>Word</code> can contain links
 * to other <code>Word</code> and they are used to model
 * depencies between different parts of sentence.
 * </p>
 * 
 * @see ucot.parser.Sentence
 */
public class Word {

    /** Original form of the word this object is carrying. */
    private String original;

    /** Basic form of the word this object is carrying. */
    private String basicForm;

    /**
     * Class of the word.
     * 
     * @see ParserInterface#NOUN
     * @see ParserInterface#VERB
     */
    private String wordClass;

    /**
     * The links from this <code>Word</code> to other
     * words.
     */
    private List<Link> links;

    /**
     * <p>
     * Constructs the word object.
     * </p>
     * 
     * @param basicForm
     *            The basic form of the word.
     * @param wordClass
     *            The class of the word.
     */
    public Word(String basicForm, String wordClass) {
        this.basicForm = basicForm;
        this.original = basicForm;
        this.wordClass = wordClass;
        links = new Vector<Link>();
    }

    /**
     * <p>
     * Adds link..
     * </p>
     * 
     * @param l
     *            The link to be added.
     */
    public void addLink(Link l) {
        links.add(l);
    }

    /**
     * <p>
     * Returns all the links. Modifying the link list does
     * not change the true linkages.
     * </p>
     * 
     * @return List of links.
     */
    public List<Link> getLinks() {
        List<Link> ret = new Vector<Link>();
        ret.addAll(links);
        return ret;
    }

    /**
     * <p>
     * Returns links of spesific name. Modification of the
     * returned list does not change the true linkages.
     * </p>
     * 
     * @param name
     *            The link name we are interested in.
     * @return The list of the links called by the given
     *         name.
     */
    public List<Link> getLinks(String name) {
        List<Link> ret = new Vector<Link>();
        for (Link link : links) {
            if (link.getName().equals(name)) {
                ret.add(link);
            }
        }
        return ret;
    }

    /**
     * <p>
     * Returns the first link called by spesific name-
     * </p>
     * 
     * @param name
     *            The name
     * @return The first link called by the given name.
     */
    public Link getLink(String name) {
        List<Link> links = getLinks(name);
        if (links.size() > 0) {
            return links.get(0);
        } else {
            return null;
        }

    }

    /**
     * <p>
     * Returns the word that is refered by the first link
     * called by the given name.
     * </p>
     * 
     * @param name
     *            The name.
     * @return The word of refered from the first link.
     */
    public Word getLinked(String name) {
        Link link = getLink(name);
        return null == link ? null : link.getTo();

    }

    /**
     * <p>
     * Removes link from this word.
     * </p>
     * 
     * @param l
     *            The link to be removed.
     * @return True if the link was contained by this word.
     */
    public boolean RemoveLink(Link l) {
        return links.remove(l);
    }

    /**
     * <p>
     * Returns the basic form of the word.
     * </p>
     * 
     * @return the basic form of the word.
     */
    public String getBasicForm() {
        return basicForm;
    }

    /**
     * <p>
     * Sets the basic form of the word.
     * </p>
     * 
     * @param basicForm
     *            The basic form.
     */
    public void setBasicForm(String basicForm) {
        this.basicForm = basicForm;
    }

    /**
     * <p>
     * Returns the word's class (part of the speech: noun,
     * verb and so on).
     * </p>
     * 
     * @return The word's class.
     */
    public String getWordClass() {
        return wordClass;
    }

    /**
     * <p>
     * Sets the words class (part of the speech: noun, verb
     * and so on).
     * </p>
     * 
     * @param wordClass
     *            The word's class.
     */
    public void setWordClass(String wordClass) {
        this.wordClass = wordClass;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append(this.basicForm + " ("
                + this.wordClass);
        if (links.size() > 0) buffer.append(" " + links);
        buffer.append(")");

        return buffer.toString();
    }

    /**
     * <p>
     * Returns the original form of the word.
     * </p>
     * 
     * @return The original form of the word.
     */
    public String getOriginal() {
        return original;
    }

    /**
     * <p>
     * Sets the original form of the word.
     * </p> *
     */
    public void setOriginal(String original) {
        this.original = original;
    }

}
