/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui;

/**
 * <p>
 * Interface for a progress bar used in core component to
 * indicate the status of parsing and running the heuristic.
 * </p>
 * 
 * <p>
 * This interface defines all methods required for UCOT core
 * to show progress bars for different slow operations. It
 * is up to the user interface itself to decide whether the
 * showing of the progress bar should disable other
 * functionality of user interface or not, but in either
 * case the progress bar is always spawned from a separate
 * thread in the UCOT core. This way the total jamming of
 * the user interface during slow operations is avoided.
 * </p>
 * 
 * @author tujupien
 */
public interface ProgressBarInterface {

    /**
     * <p>
     * Gets the maximum value for the progress bar.
     * </p>
     * 
     * @return The maximum value for the progress bar.
     */
    public int getMaximum();

    /**
     * <p>
     * Gets the minimum value for the progress bar.
     * </p>
     * 
     * @return The minimum value for the progress bar.
     */
    public int getMinimum();

    /**
     * <p>
     * Gets the current percentage completed. Which is
     * <code>getValue() / (getMaximum() - getMinimum())</code>.
     * </p>
     * 
     * @return Current percentage completed.
     */
    public double getPercentageComplete();

    /**
     * <p>
     * Gets the current action description.
     * </p>
     * 
     * @return Action description.
     */
    public String getString();

    /**
     * <p>
     * Gets the current value of the progress bar.
     * </p>
     * 
     * @return Current value.
     */
    public int getValue();

    /**
     * <p>
     * Sets the maximum value of the progress bar.
     * </p>
     * 
     * @param maximum
     *            Maximum value of the progress bar.
     */
    public void setMaximum(int maximum);

    /**
     * <p>
     * Sets the minimum value of the progress bar.
     * </p>
     * 
     * @param minimum
     *            Minimum value of the progress bar.
     */
    public void setMinimum(int minimum);

    /**
     * <p>
     * Sets the action description string.
     * </p>
     * 
     * @param string
     *            Current action description.
     */
    public void setString(String string);

    /**
     * <p>
     * Sets the current value of the progress bar.
     * </p>
     * 
     * @param value
     *            Current value.
     */
    public void setValue(int value);

    /**
     * <p>
     * Method for setting the progress bar (dialog) visible
     * or hiding it.
     * </p>
     * 
     * @param visible
     *            Indicates wether the progress bar is
     *            visible or invisible.
     */
    public void setVisible(boolean visible);

}
