/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Window;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.net.URL;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import ucot.utils.CustomFileFilter;

/**
 * <p>
 * Miscallaneous utils used by the grapical userinterface.
 * </p>
 * 
 * @author ilanliuk
 * 
 */
public class GUIUtils {
    protected final static String DEFAULT_ERROR_HEADER = "Error";

    protected final static String DEFAULT_QUESTION_HEADER = "Question";

    protected final static String DEFAULT_WARNING_HEADER = "Warning";

    protected final static String OK_CAPTION = "OK";

    /**
     * <p>
     * Method for spawning a file chooser dialog.
     * </p>
     * 
     * @param owner
     *            Parent of this dialog.
     * @param fileFilters
     *            Allowed file extensions.
     * @param acceptAllFileFiltersUsed
     *            Defines whether all file types are allowed
     *            or not when any file filters are manually
     *            defined.
     * @return The selected file as an URL or null if no
     *         file was selected.
     */
    public static URL showOpenFileDialog(Component owner,
            Vector<CustomFileFilter> fileFilters,
            boolean acceptAllFileFiltersUsed) {
        JFileChooser fileChooser = new JFileChooser(".");
        if (fileFilters != null) {
            fileChooser
                    .setAcceptAllFileFilterUsed(acceptAllFileFiltersUsed);
            for (FileFilter filter : fileFilters) {
                fileChooser.setFileFilter(filter);
            }
        }
        int choice = fileChooser.showOpenDialog(owner);

        URL result = null;

        if (choice == JFileChooser.APPROVE_OPTION) {
            try {
                result = fileChooser.getSelectedFile()
                        .toURL();
            } catch (Exception e) {
                System.err.println(e.getMessage());
                System.err.println(e.getStackTrace()
                        .toString());
            }
        }
        return result;
    }

    /**
     * <p>
     * Default open dialog spawner which assumes that all
     * file types are not required to be shown when any file
     * filters are manually defined.
     * </p>
     * 
     * @param owner
     *            Parent of this dialog.
     * @param fileFilters
     *            Allowed file extensions.
     * @return The selected file as an URL or null if no
     *         file was selected.
     */
    public static URL showOpenFileDialog(Component owner,
            Vector<CustomFileFilter> fileFilters) {
        return showOpenFileDialog(owner, fileFilters, false);
    }

    /**
     * <p>
     * Method for spawning a file save dialog.
     * </p>
     * 
     * @param owner
     *            Parent of this dialog.
     * @param fileFilters
     *            Allowed file extensions.
     * @param fileSelectionMode
     *            defines wether the user can select files,
     *            directories or both.
     * @return The object containing selections.
     */
    public static ChoosedFile showSaveFileDialog(
            Component owner,
            Vector<CustomFileFilter> fileFilters,
            int fileSelectionMode) {
        JFileChooser fileChooser = new JFileChooser(".");
        if (fileFilters != null) {
            fileChooser.setAcceptAllFileFilterUsed(false);
            for (FileFilter filter : fileFilters) {
                fileChooser.setFileFilter(filter);
            }
        }

        fileChooser.setFileSelectionMode(fileSelectionMode);
        int choice = fileChooser.showSaveDialog(owner);

        URL result = null;

        if (choice == JFileChooser.APPROVE_OPTION) {
            try {
                // Force file extension to be valid.
                CustomFileFilter filter = null;
                try {
                    filter = (CustomFileFilter) fileChooser
                            .getFileFilter();
                } catch (Exception e) {
                }

                File resultFile = fileChooser
                        .getSelectedFile();
                // TODO check this.
                if (filter != null
                        && !filter.accept(resultFile)) {
                    resultFile = new File(resultFile
                            .getAbsolutePath()
                            + "."
                            + filter.getExtensions()
                                    .firstElement());
                }
                result = resultFile.toURL();
            } catch (Exception e) {
                e.printStackTrace();
                System.err.println(e.getMessage());
                System.err.println(e.getStackTrace()
                        .toString());
            }
        }

        // If this was called without customfilefilters
        // filechooser.getFileFilter() returns
        // AcceptAllFileFilter
        // ..so we need to create own filter for this
        CustomFileFilter usedFilter = (null != fileFilters)
                ? (CustomFileFilter) fileChooser
                        .getFileFilter()
                : new CustomFileFilter("All Files",
                        new String[] { "all" });

        return new ChoosedFile(result, usedFilter);
    }

    /**
     * <p>
     * Method for spawning a save file chooser dialog.
     * </p>
     * 
     * @param owner
     *            Parent of this dialog.
     * @param fileFilters
     *            Allowed file extensions.
     * @return The selected file as an URL or null if no
     *         file was selected.
     */
    public static ChoosedFile showSaveFileDialog(
            Component owner,
            Vector<CustomFileFilter> fileFilters) {
        return showSaveFileDialog(owner, fileFilters,
                JFileChooser.FILES_ONLY);
    }

    /**
     * <p>
     * Dialog spawner for warnings, errors and questions.
     * </p>
     * 
     * @param parent
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param messageType
     *            the type of message to be displayed:
     *            <code>ERROR_MESSAGE</code>,
     *            <code>INFORMATION_MESSAGE</code>,
     *            <code>WARNING_MESSAGE</code>,
     *            <code>QUESTION_MESSAGE</code>, or
     *            <code>PLAIN_MESSAGE</code>.
     * @param message
     *            Message itself.
     * @param header
     *            the <code>String</code> to display in
     *            the dialog's title bar
     * @param options
     *            the options to display in the pane:
     *            <code>DEFAULT_OPTION</code>,
     *            <code>YES_NO_OPTION</code>,
     *            <code>YES_NO_CANCEL_OPTION, or 
     * 		<code>OK_CANCEL_OPTION</code>.
     * @return User's choice.
     */
    public static int showDialog(Window parent,
            int messageType, String message, String header,
            int options) {
        final JOptionPane optionPane = new JOptionPane(
                message, messageType, options);
        final JDialog dialog = new JDialog(new JFrame(),
                header, true);

        // Initialize question dialog.
        dialog.setContentPane(optionPane);
        dialog
                .setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        dialog.pack();
        dialog.setResizable(false);
        optionPane
                .addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(
                            PropertyChangeEvent e) {
                        String prop = e.getPropertyName();

                        if (dialog.isVisible()
                                && (e.getSource() == optionPane)
                                && (prop
                                        .equals(JOptionPane.VALUE_PROPERTY))) {
                            dialog.setVisible(false);
                        }
                    }
                });

        // Spawn question dialog and return user's choice.
        centerDialog(dialog);
        dialog.setAlwaysOnTop(true);
        dialog.setVisible(true);
        return (Integer) optionPane.getValue();
    }

    /**
     * <p>
     * This method centers a given dialog to the center of
     * the screen.
     * </p>
     * 
     * @param dialog
     *            <code>JDialog</code> to be centered.
     */
    public static void centerDialog(JDialog dialog) {
        Dimension screenSize = dialog.getToolkit()
                .getScreenSize();
        Dimension dialogSize = dialog.getSize();
        Point location = new Point();
        location.setLocation(Math.max(Math
                .floor((screenSize.getWidth() - dialogSize
                        .getWidth()) / 2), 0),
                Math
                        .max(Math.floor((screenSize
                                .getHeight() - dialogSize
                                .getHeight()) / 2), 0));
        dialog.setLocation(location);
    }

    /**
     * <p>
     * Method for spawning question dialog.
     * </p>
     * 
     * @param win
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param question
     *            The question <code>String</code> to
     *            show.
     * @return User's choice.
     */
    public static int questionDialog(Window win,
            String question) {
        return questionDialog(win, question,
                DEFAULT_QUESTION_HEADER);
    }

    /**
     * <p>
     * Method for spawning warning dialog.
     * </p>
     * 
     * @param win
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param warning
     *            The warning <code>String</code> to show.
     * @param header
     *            the <code>String</code> to display in
     *            the dialog's title bar
     */
    public static void printWarning(Window win,
            String warning, String header) {
        showDialog(win, JOptionPane.WARNING_MESSAGE,
                warning, header, -1);
    }

    /**
     * <p>
     * Method for spawning info dialog.
     * </p>
     * 
     * @param win
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param info
     *            The info <code>String</code> to show.
     * @param header
     *            the <code>String</code> to display in
     *            the dialog's title bar
     */
    public static void printInfo(Window win, String info,
            String header) {
        showDialog(win, JOptionPane.INFORMATION_MESSAGE,
                info, header, -1);
    }

    /**
     * <p>
     * Method for spawning warning dialog with dafault
     * header <code>String</code>.
     * </p>
     * 
     * @param win
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param warning
     *            The warning <code>String</code> to show.
     */
    public static void printWarning(Window win,
            String warning) {
        printWarning(win, warning, DEFAULT_WARNING_HEADER);
    }

    /**
     * <p>
     * Method for spawning error dialog.
     * </p>
     * 
     * @param win
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param errorMessage
     *            The error <code>String</code> to show.
     * @param header
     *            the <code>String</code> to display in
     *            the dialog's title bar
     */
    public static void printError(Window win,
            String errorMessage, String header) {
        showDialog(win, JOptionPane.ERROR_MESSAGE,
                errorMessage, header, -1);
    }

    /**
     * <p>
     * Method for spawning error dialog with default header
     * <code>String</code>
     * </p>
     * 
     * @param win
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param errorMessage
     *            The error <code>String</code> to show.
     */
    public static void printError(Window win,
            String errorMessage) {
        printError(win, errorMessage, DEFAULT_ERROR_HEADER);
    }

    /**
     * <p>
     * Method for spawning question dialog with wanted
     * options.
     * </p>
     * <p>
     * Options available are: <code>DEFAULT_OPTION</code>,
     * <code>YES_NO_OPTION</code>,
     * <code>YES_NO_CANCEL_OPTION, or 
     * 		<code>OK_CANCEL_OPTION</code>.
     * Options are defined in <code>JOptionPane</code>-class.
     * </p>
     * 
     * @param win Parent <code>Window</code> for this dialog.
     * @param question The question <code>String</code> to show.
     * @param header the <code>String</code> to display in the dialog's title bar
     * @param options The options to display in the pane.
     * @return User's choise.
     * 
     * @see JOptionPane
     */
    public static int questionDialog(Window win,
            String question, String header, int options) {
        return showDialog(win,
                JOptionPane.QUESTION_MESSAGE, question,
                header, options);
    }

    /**
     * <p>
     * Method for spawning question dialog with
     * <code>YES_NO_OPTION</code>-option.
     * </p>
     * 
     * @param win
     *            Parent <code>Window</code> for this
     *            dialog.
     * @param question
     *            The question <code>String</code> to
     *            show.
     * @param header
     *            the <code>String</code> to display in
     *            the dialog's title bar
     * @return User's choise.
     */
    public static int questionDialog(Window win,
            String question, String header) {
        return questionDialog(win, question, header,
                JOptionPane.YES_NO_OPTION);
    }

    /**
     * <p>
     * This method creates a new JComboBox Cell Editor for a
     * JTable element from the given String array. Duplicate
     * entries from given array will be filtered and the
     * array is sorted to alphabetical order.
     * </p>
     * 
     * @param values
     *            Available options in the JComboBox.
     * @param allowSelf
     *            Allow self pointer in the list.
     * @param self
     *            The name of the self object.
     * @param selfPointer
     *            The name of the pointer used to point the
     *            self object.
     * @param addEmpty
     *            Create an empty item to the beginning of
     *            the list.
     * @return JComboBox table cell editor as a
     *         DefaultCellEditor.
     */
    public static DefaultCellEditor createComboBoxCellEditor(
            String[] values, String self,
            String selfPointer, boolean allowSelf,
            boolean addEmpty) {
        JComboBox result = new JComboBox();
        List<String> valuesList = Arrays.asList(values);
        // Sort values to alphabetical order.
        Collections.sort(valuesList);
        // Create self pointer and empty items to the
        // beginning of the list
        // if required.
        if (addEmpty) result.addItem("");
        if (allowSelf) result.addItem(selfPointer);
        // Add values to the JComboBox item list and filter
        // out duplicates.
        for (int i = 0; i < valuesList.size(); i++) {
            if (valuesList.get(i).equals(self)) continue;
            if (i >= 0
                    && result.getItemCount() > 0
                    && !valuesList.get(i).equals(
                            result.getItemAt(result
                                    .getItemCount() - 1))) result
                    .addItem(valuesList.get(i));
            else if (result.getItemCount() == 0) result
                    .addItem(valuesList.get(i));
        }
        // Spawn a new DefaultCellEditor from the resulting
        // JComboBox.
        return new DefaultCellEditor(result);
    }

}
