/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.dialog;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ucot.Messages;
import ucot.core.ControlInterface;
import ucot.heuristic.HeuristicInterface;
import ucot.parser.ParserInterface;

/**
 * <p>
 * Dialog extending <code>JDialog</code> for asking from
 * user wich <code>ParserInterface</code> and
 * <code>HeuristicInterface</code> he/she wants to use.
 * </p>
 * 
 * <p>
 * Selected <code>ParserInterface</code> and
 * <code>HeuristicInterface</code> can be resolved with
 * static methods <code>getParser()</code> and
 * <code>getHeuristic()</code>. If user closed dialog
 * without selecting <code>ParserInterface</code> and
 * <code>HeuristicInterface</code> methods
 * <code>getParser()</code> and
 * <code>getHeuristic()</code> return <code>null</code>.
 * 
 * @see JDialog
 * 
 * @author ilanliuk
 * 
 */
public class AddToModelWithDialog extends JDialog {

    private ControlInterface core;

    private static final long serialVersionUID = 8969289172710583832L;

    private static ParserInterface selectedParser = null;

    private static HeuristicInterface selectedHeuristic = null;

    //
    // Dialog components
    private JLabel parserLabel = new JLabel(
            Messages
                    .getString("AddToModelWithDialog.SELECT_PARSER")); // 

    private JComboBox parserComboBox = new JComboBox();

    private JLabel heuristicLabel = new JLabel(
            Messages
                    .getString("AddToModelWithDialog.SELECT_HEURISTIC")); //$NON-NLS-1$

    private JComboBox heuristicComboBox = new JComboBox();

    // Dialog components
    //

    /**
     * <p>
     * Default constructor for
     * <code>AddToModelWithDialog</code>.
     * </p>
     * 
     * @param owner
     *            the <code>Frame</code> from which the
     *            dialog is displayed.
     * @param core
     *            <code>ControllInterface</code>.
     * @throws HeadlessException
     *             if GraphicsEnvironment.isHeadless()
     *             returns true.
     */
    public AddToModelWithDialog(Frame owner,
            ControlInterface core) throws HeadlessException {
        super(
                owner,
                Messages
                        .getString("AddToModelWithDialog.ADD_TO_MODEL_WITH"),
                true);

        this.core = core;

        this.setLayout(new BorderLayout());

        JPanel parserPanel = new JPanel();
        parserPanel.setLayout(new FlowLayout(
                FlowLayout.CENTER, 10, 10));
        parserPanel.add(parserLabel);
        parserPanel.add(parserComboBox);

        JPanel heuristicPanel = new JPanel();
        heuristicPanel.setLayout(new FlowLayout(
                FlowLayout.CENTER, 10, 10));
        heuristicPanel.add(heuristicLabel);
        heuristicPanel.add(heuristicComboBox);

        // set up ComboBoxes
        InitializeComboBoxes();

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());

        JButton okButton = new JButton(
                Messages
                        .getString("AddToModelWithDialog.OK_BUTTON_CAPTION"));
        // Create actionlistener for okButton
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                buttonOKClicked();
            }
        });
        buttonPanel.add(okButton);

        JButton cancelButton = new JButton(
                Messages
                        .getString("AddToModelWithDialog.CANCEL_BUTTON_CAPTION"));
        // Create actionlistener for cancelButton
        cancelButton
                .addActionListener(new ActionListener() {
                    public void actionPerformed(
                            ActionEvent e) {
                        buttonCancelClicked();
                    }
                });
        buttonPanel.add(cancelButton);

        this.getContentPane().add(parserPanel,
                BorderLayout.NORTH);
        this.getContentPane().add(heuristicPanel,
                BorderLayout.CENTER);
        this.getContentPane().add(buttonPanel,
                BorderLayout.SOUTH);

        this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        this.addWindowListener(dialogListener);

        this.setResizable(false);
        this.pack();
    }

    /**
     * <p>
     * Method sets values of <code>selectedParser</code>
     * and <code>selectedHeuristic</code> same that are in
     * <code>parserComboBox</code> and
     * <code>heuristicComboBox</code> and closes dialog.
     * </p>
     * 
     */
    private void buttonOKClicked() {
        Object o = parserComboBox.getSelectedItem();

        selectedParser = (ParserInterface) o;

        o = heuristicComboBox.getSelectedItem();
        selectedHeuristic = (HeuristicInterface) o;

        setVisible(false);
    }

    /**
     * <p>
     * Method sets values of <code>selectedParser</code>
     * and <code>selectedHeuristic</code> to
     * <code>null</code> and closes dialog.
     * </p>
     * 
     */
    private void buttonCancelClicked() {

        // set selected parser and heuristic null
        // 
        selectedHeuristic = null;
        selectedParser = null;

        setVisible(false);
    }

    /**
     * Custom WindowListener to override
     * windowClosing-event. Action in windowClosing-event is
     * same as when user presses Cancel-button.
     */
    private WindowListener dialogListener = new WindowAdapter() {

        @Override
        public void windowClosing(WindowEvent e) {
            buttonCancelClicked();
        }

    };

    /**
     * <p>
     * Sets up <code>parserComboBox</code> and
     * <code>heuristicComboBox</code>. Clears items from
     * them and adds new items
     * </p>.
     */
    private void InitializeComboBoxes() {

        parserComboBox.removeAllItems();
        heuristicComboBox.removeAllItems();

        // set up parserComboBox
        List<ParserInterface> parsers = core.getParsers();
        for (ParserInterface parser : parsers) {
            parserComboBox.addItem(parser);
        }
        // set default parser as default selection of
        // combobox
        parserComboBox.setSelectedItem(core
                .getCurrentParser());

        // set up heuristicComboBox
        List<HeuristicInterface> heuristics = core
                .getHeuristics();

        for (HeuristicInterface heuristic : heuristics) {
            heuristicComboBox.addItem(heuristic);
        }
        // set default heuristic as default selection of
        // combobox
        heuristicComboBox.setSelectedItem(core
                .getCurrentHeuristic());

    }

    /**
     * <p>
     * Static method to create and show dialog. Returns true
     * if user clicked OK button in dialog else returns
     * false.
     * </p>
     * 
     * @param core
     *            <code>ControlInterface</code>
     * @return <code>true</code> if user clicked OK-button
     *         in dialog, else returns <code>false</code>.
     */
    public static boolean showDialog(ControlInterface core) {

        // create new dialog for selecting parser and
        // heuristic to use
        AddToModelWithDialog dialog = new AddToModelWithDialog(
                null, core);

        // show dialog
        dialog.setVisible(true);

        // if parser and heuristic are selected (user
        // pressed OK)
        // return true if not return false
        if (selectedHeuristic != null
                && selectedParser != null) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * <p>
     * Returns <code>HeuristicInterface</code> that user
     * selected, or <code>null</code> if no
     * <code>HeuristicInterface</code> was selected.
     * <p>
     * 
     * @return <code>HeuristicInterface</code> user
     *         selected from dialog.
     */
    public static HeuristicInterface getHeuristic() {
        return selectedHeuristic;
    }

    /**
     * Returns <code>ParserInterface</code> that user
     * selected, or <code>null</code> if no
     * <code>ParserInterface</code> was selected.
     * 
     * @return <code>ParserInterface</code> user selected
     *         from dialog.
     */
    public static ParserInterface getParser() {
        return selectedParser;
    }

}
