/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import ucot.Messages;
import ucot.core.ControlInterface;
import ucot.exceptions.BadPropertyValueException;
import ucot.heuristic.HeuristicInterface;
import ucot.model.ModelEditor;
import ucot.parser.ParserInterface;
import ucot.ui.gui.GUIUtils;
import ucot.ui.gui.GraphicalUI;
import ucot.ui.gui.dot.DotColorModel;
import ucot.utils.Sets;
import ucot.utils.StringTools;

/**
 * <p>
 * Settings dialog for UCOT program. There are different
 * kinds of settings available for modification through UCOT
 * (G)UI, and this dialog allows user to change the values
 * for those settings.
 * </p>
 * 
 * <p>
 * This settings dialog uses the ModulePropertyInterface
 * offered by the UCOT modules and after changing the values
 * each component's <tt>applyProperties</tt> method is
 * called.
 * </p>
 * 
 * @author ilanliuk, tujupien
 */

public class SettingsDialog extends JDialog {

    private static final long serialVersionUID = 3186714875647715870L;

    private GraphicalUI owner;

    private Logger logger = Logger.getLogger("ucot.ui");

    private ActionListener buttonListener;

    private String result = MAIN_CANCEL_BUTTON_ACTION;

    private JComboBox parserComboBox;

    private JComboBox heuristicComboBox;

    private JList typeList;

    private DefaultListModel typeListModel;

    private Vector<JTextField> fileTextFields = new Vector<JTextField>();

    private Vector<JButton> fileBrowserButtons = new Vector<JButton>();

    private int dotPathIndex;

    private int epsToPDFpathIndex;

    private Vector<JComboBox> graphComboBoxes = new Vector<JComboBox>();

    private int normalColorIndex;

    private int highlightColorIndex;

    // Action commands for buttons.
    public static final String MAIN_OK_BUTTON_ACTION = "BUTTON_OK_CLICKED";

    public static final String MAIN_CANCEL_BUTTON_ACTION = "BUTTON_CANCEL_CLICKED";

    private static final String BROWSE_BUTTON_CLICKED = "BUTTON_BROWSE_CLICKED";

    private static final int HORIZONTAL_GAP = 10;

    private static final int VERTICAL_GAP = 5;

    // Static strings.
    public static final String DIALOG_TITLE = Messages
            .getString("SettingsDialog.TITLE");

    public static final String MAIN_OK_BUTTON = Messages
            .getString("SettingsDialog.MAIN_OK_BUTTON");

    public static final String MAIN_CANCEL_BUTTON = Messages
            .getString("SettingsDialog.MAIN_CANCEL_BUTTON");

    public static final String BROWSE_BUTTON_TITLE = Messages
            .getString("SettingsDialog.BROWSE_BUTTON");

    public static final String DOT_PATH_LABEL = Messages
            .getString("SettingsDialog.DOT_PATH_LABEL");

    public static final String EPS_TO_PDF_PATH_LABEL = Messages
            .getString("SettingsDialog.EPS_TO_PDF_PATH_LABEL");

    public static final String EXTERNAL_FILES_TITLE = Messages
            .getString("SettingsDialog.EXTERNAL_FILES_TITLE");

    public static final String GENERAL_SETTINGS_TITLE = Messages
            .getString("SettingsDialog.GENERAL_SETTINGS_TITLE");

    public static final String GRAPH_SETTINGS_TITLE = Messages
            .getString("SettingsDialog.GRAPH_SETTINGS_TITLE");

    public static final String ENTITY_TYPE_TITLE = Messages
            .getString("SettingsDialog.ENTITY_TYPE_TITLE");

    public static final String DEFAULT_PARSER_LABEL = Messages
            .getString("SettingsDialog.DEFAULT_PARSER_LABEL");

    public static final String DEFAULT_HEURISTIC_LABEL = Messages
            .getString("SettingsDialog.DEFAULT_HEURISTIC_LABEL");

    public static final String INCORRECT_PATH_TO_FILE_QUESTION = Messages
            .getString("SettingsDialog.INCORRECT_PATH_TO_FILE_QUESTION");

    public static final String DOT_COLOR_LABEL = Messages
            .getString("SettingsDialog.DOT_COLOR_LABEL");

    public static final String DOT_HIGHLIGHT_COLOR_LABEL = Messages
            .getString("SettingsDialog.DOT_HIGHLIGHT_COLOR_LABEL");

    public static final String ENTITY_TYPES_GOING_TO_BE_REMOVED_QUESTION = Messages
            .getString("SettingsDialog.ENTITY_TYPES_GOING_TO_BE_REMOVED_QUESTION");

    /**
     * <p>
     * Default constructor for SettingsDialog. This
     * constructor initializes the whole dialog, creates the
     * layout and makes all defined properties available for
     * modification.
     * </p>
     * 
     * @param owner
     *            GraphicalUI that owns this dialog.
     * @throws If
     *             something with the initialization of the
     *             (super) class goes wrong, a
     *             HeadlessException is thrown.
     */
    public SettingsDialog(GraphicalUI owner)
            throws HeadlessException {
        // Initialize some class variables.
        super(owner, DIALOG_TITLE, true);
        this.owner = owner;

        // Create panel for settings. Later this could
        // include
        // different tabs, but for now it is not necessary
        // due
        // to the ridiculously low number of different
        // settings
        // available.
        this.getContentPane().setLayout(
                new BoxLayout(getContentPane(),
                        BoxLayout.Y_AXIS));

        // Create a panel for buttons.
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(
                FlowLayout.CENTER));

        // initialize actionlistener for buttons
        this.createButtonListener();

        // Initialize different kinds of panels for
        // settings.
        getContentPane().add(createGeneralPanel());
        getContentPane().add(createExternalFilesPanel());
        getContentPane().add(createGraphSettingsPanel());
        getContentPane().add(createEntityTypesPanel());

        // Generate buttons.
        JButton button = new JButton(MAIN_OK_BUTTON);
        button.setActionCommand(MAIN_OK_BUTTON_ACTION);
        button.addActionListener(buttonListener);
        buttonPanel.add(button);

        button = new JButton(MAIN_CANCEL_BUTTON);
        button.setActionCommand(MAIN_CANCEL_BUTTON_ACTION);
        button.addActionListener(buttonListener);
        buttonPanel.add(button);

        this.getContentPane().add(buttonPanel,
                BorderLayout.SOUTH);

        this.pack();
        this.setResizable(false);
    }

    // TODO comment me
    private JPanel createGraphSettingsPanel() {
        JPanel result = new JPanel();
        result.setBorder(BorderFactory
                .createTitledBorder(GRAPH_SETTINGS_TITLE));
        result.setLayout(new BoxLayout(result,
                BoxLayout.Y_AXIS));

        JPanel row = new JPanel();
        row.setLayout(new BorderLayout(HORIZONTAL_GAP,
                VERTICAL_GAP));

        // Dot color:
        row.add(new JLabel(DOT_COLOR_LABEL),
                BorderLayout.WEST);

        JComboBox colorComboBox = new JComboBox();

        for (DotColorModel.ColorModel color : DotColorModel.ColorModel
                .values()) {
            colorComboBox.addItem(color.toString()
                    .toLowerCase().replace('_', ' '));
        }
        colorComboBox
                .setSelectedItem(owner.getColorTheme()
                        .getColor().toLowerCase().replace(
                                '_', ' '));
        row.add(colorComboBox, BorderLayout.CENTER);
        graphComboBoxes.add(colorComboBox);
        normalColorIndex = graphComboBoxes.size() - 1;
        result.add(row);

        // Dot highlight color:
        row = new JPanel();
        row.setLayout(new BorderLayout(HORIZONTAL_GAP,
                VERTICAL_GAP));
        row.add(new JLabel(DOT_HIGHLIGHT_COLOR_LABEL),
                BorderLayout.WEST);

        colorComboBox = new JComboBox();

        for (DotColorModel.ColorModel color : DotColorModel.ColorModel
                .values()) {
            colorComboBox.addItem(color.toString()
                    .toLowerCase().replace('_', ' '));
        }
        colorComboBox.setSelectedItem(owner.getColorTheme()
                .getHighlight().toLowerCase().replace('_',
                        ' '));
        row.add(colorComboBox, BorderLayout.CENTER);
        graphComboBoxes.add(colorComboBox);
        highlightColorIndex = graphComboBoxes.size() - 1;
        result.add(row);

        return result;
    }

    /**
     * <p>
     * This method creates a panel containing text boxes and
     * browse buttons for modifying the path all external
     * files that are associated with this program.
     * </p>
     */
    private JPanel createExternalFilesPanel() {

        JPanel result = new JPanel();
        result.setBorder(BorderFactory
                .createTitledBorder(EXTERNAL_FILES_TITLE));
        result.setLayout(new BoxLayout(result,
                BoxLayout.Y_AXIS));

        JPanel row = new JPanel();
        row.setLayout(new BorderLayout(HORIZONTAL_GAP,
                VERTICAL_GAP));

        // Modifier for DOT PATH
        row.add(new JLabel(DOT_PATH_LABEL),
                BorderLayout.WEST);

        JTextField textField = new JTextField(owner
                .getProperties().getProperty(
                        GraphicalUI.PROPERTY_DOT_PATH));
        row.add(textField, BorderLayout.CENTER);

        JButton pathBrowseButton = new JButton(
                BROWSE_BUTTON_TITLE);
        pathBrowseButton
                .setActionCommand(BROWSE_BUTTON_CLICKED);
        pathBrowseButton.addActionListener(buttonListener);
        row.add(pathBrowseButton, BorderLayout.EAST);

        fileTextFields.add(textField);
        fileBrowserButtons.add(pathBrowseButton);
        dotPathIndex = this.fileTextFields.size() - 1;
        result.add(row);

        // Modifier for EPS to PDF PATH
        row = new JPanel();
        row.setLayout(new BorderLayout(HORIZONTAL_GAP,
                VERTICAL_GAP));
        row.add(new JLabel(EPS_TO_PDF_PATH_LABEL),
                BorderLayout.WEST);

        textField = new JTextField(
                owner
                        .getProperties()
                        .getProperty(
                                GraphicalUI.PROPERTY_EPS_TO_PDF_PATH));
        row.add(textField, BorderLayout.CENTER);

        pathBrowseButton = new JButton(BROWSE_BUTTON_TITLE);
        pathBrowseButton
                .setActionCommand(BROWSE_BUTTON_CLICKED);
        pathBrowseButton.addActionListener(buttonListener);
        row.add(pathBrowseButton, BorderLayout.EAST);

        this.fileTextFields.add(textField);
        this.fileBrowserButtons.add(pathBrowseButton);
        this.epsToPDFpathIndex = this.fileTextFields.size() - 1;
        result.add(row);

        return result;
    }

    /**
     * <p>
     * Initialize panel for general settings, like default
     * parser and heuristics etc.
     * </p>
     */
    private JPanel createGeneralPanel() {

        JPanel result = new JPanel();
        result
                .setBorder(BorderFactory
                        .createTitledBorder(GENERAL_SETTINGS_TITLE));
        result.setLayout(new BoxLayout(result,
                BoxLayout.Y_AXIS));

        JPanel row = new JPanel();
        row.setLayout(new BorderLayout(HORIZONTAL_GAP,
                VERTICAL_GAP));

        // Default parser:
        row.add(new JLabel(DEFAULT_PARSER_LABEL),
                BorderLayout.WEST);

        // Create a combo box for parser selection.
        parserComboBox = new JComboBox();
        List<ParserInterface> parsers = owner
                .getControlInterface().getParsers();
        for (ParserInterface parser : parsers) {
            parserComboBox.addItem(parser);
        }
        parserComboBox.setSelectedItem(owner
                .getControlInterface().getCurrentParser());
        row.add(parserComboBox, BorderLayout.CENTER);
        result.add(row);

        // Default heuristic:
        row = new JPanel();
        row.setLayout(new BorderLayout(HORIZONTAL_GAP,
                VERTICAL_GAP));
        row.add(new JLabel(DEFAULT_HEURISTIC_LABEL),
                BorderLayout.WEST);

        // Create ComboBox for selecting default heuristic
        heuristicComboBox = new JComboBox();
        List<HeuristicInterface> heuristics = owner
                .getControlInterface().getHeuristics();
        for (HeuristicInterface heuristic : heuristics) {
            heuristicComboBox.addItem(heuristic);
        }
        heuristicComboBox.setSelectedItem(owner
                .getControlInterface()
                .getCurrentHeuristic());
        row.add(heuristicComboBox, BorderLayout.CENTER);
        result.add(row);

        return result;
    }

    /**
     * <p>
     * Initialize <code>JPanel</code> for entity types.
     * </p>
     * 
     * @return <code>JPanel</code>
     */
    private JPanel createEntityTypesPanel() {
        JPanel result = new JPanel();
        result.setBorder(BorderFactory
                .createTitledBorder(ENTITY_TYPE_TITLE));
        result.setLayout(new FlowLayout());

        // create panels for JList and buttons..
        JPanel listPane = new JPanel();
        JScrollPane scrollListPane = new JScrollPane(
                listPane);
        listPane.setLayout(new BorderLayout());
        scrollListPane.setPreferredSize(new Dimension(90,
                110));
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new BoxLayout(buttonPane,
                BoxLayout.Y_AXIS));

        result.add(buttonPane);
        result.add(scrollListPane);

        typeListModel = new DefaultListModel();
        // populate the listmodel with entity types
        for (String type : this.owner.entityTypes) {
            typeListModel.addElement(type);
        }

        // create list of entity types
        typeList = new JList(typeListModel);
        typeList.setLayoutOrientation(JList.VERTICAL);
        typeList.setVisibleRowCount(7);

        final JTextField newTypeTextField = new JTextField(
                15);
        buttonPane.add(newTypeTextField);
        JButton addNewType = new JButton("Add new type");
        addNewType.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {

                String newType = newTypeTextField.getText();

                if (!"".equals(newType.trim())) {
                    typeListModel.addElement(newType);
                    newTypeTextField.setText("");
                }
            }
        });

        buttonPane.add(addNewType);
        JButton removeType = new JButton("Remove from list");
        removeType.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (typeList.getSelectedIndex() != -1) {
                    typeListModel.remove(typeList
                            .getSelectedIndex());
                }
            }
        });

        buttonPane.add(removeType);

        listPane.add(typeList);

        return result;
    }

    // TODO comment me
    private void updateEntityTypes() {
        Set<String> types = new HashSet<String>();

        for (int i = 0; i < typeListModel.getSize(); i++) {
            types.add((String) typeListModel.get(i));
        }

        Set<String> newTypes = Sets.missing(
                this.owner.entityTypes, types);
        Set<String> deletedTypes = Sets.missing(types,
                this.owner.entityTypes);

        ModelEditor me = owner.getControlInterface()
                .getAnalyzeModel().getEditor();
        Set<String> entities = me.getEntityNames();
        Set<String> affectedEntities = new HashSet<String>();

        for (String entity : entities) {
            String type = me.getEntityType(entity);
            if (deletedTypes.contains(type)) {
                affectedEntities.add(entity);
            }
        }

        int option = JOptionPane.YES_OPTION;
        if (affectedEntities.size() > 0) {
            option = (Integer) GUIUtils
                    .questionDialog(
                            this,
                            Messages
                                    .formatMessage(
                                            ENTITY_TYPES_GOING_TO_BE_REMOVED_QUESTION,
                                            affectedEntities
                                                    .size()));

        }
        if (option == JOptionPane.YES_OPTION) {
            this.owner.entityTypes.removeAll(deletedTypes);
            this.owner.entityTypes.addAll(newTypes);
            for (String entity : affectedEntities) {
                me.setEntityType(entity, "");
            }
        }

    }

    /**
     * <p>
     * Method for browsing files and putting the selected
     * file to the correct text box.
     * </p>
     */
    private void buttonBrowseClicked(ActionEvent e) {
        URL filePath = GUIUtils.showOpenFileDialog(this,
                null, true);
        // Ensure the existance of the given file.
        if (filePath == null) return;
        File file = new File(filePath.getFile());
        if (!file.exists()) return;

        // Set the selected file to the correct text field.
        for (int i = 0; i < fileBrowserButtons.size(); i++) {
            if (!e.getSource().equals(
                    fileBrowserButtons.get(i))) continue;
            fileTextFields.get(i).setText(
                    filePath.getFile());
            break;
        }
    }

    /**
     * <p>
     * User just clicked cancel button and we just need
     * close the dialog without saving anything.
     * </p>
     */
    private void mainCancelButtonClicked() {
        setVisible(false);
    }

    /**
     * <p>
     * User clicked Apply and we just save the settings and
     * vanish.
     * </p>
     */
    private void mainOKButtonClicked() {
        Properties properties = new Properties();

        // Default parser:
        properties.setProperty(
                ControlInterface.PROPERTY_CURRENT_PARSER,
                ((ParserInterface) parserComboBox
                        .getSelectedItem()).toString());

        // Default heuristic:
        properties
                .setProperty(
                        ControlInterface.PROPERTY_CURRENT_HEURISTIC,
                        ((HeuristicInterface) heuristicComboBox
                                .getSelectedItem())
                                .toString());

        // Apply core's properties.
        owner.getControlInterface().setProperties(
                properties);
        try {
            owner.getControlInterface().applyProperties();
        } catch (BadPropertyValueException e) {
            logger.warning(e.getMessage());
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        properties = new Properties();

        // Dot path:
        File file = new File(fileTextFields.get(
                dotPathIndex).getText());
        if (file.exists()) {
            properties.setProperty(
                    GraphicalUI.PROPERTY_DOT_PATH,
                    fileTextFields.get(dotPathIndex)
                            .getText());
        } else {
            Object answer = GUIUtils
                    .questionDialog(
                            this,
                            Messages
                                    .formatMessage(
                                            INCORRECT_PATH_TO_FILE_QUESTION,
                                            file));
            if (answer.equals(JOptionPane.NO_OPTION)) return;
        }

        // EPS to PDF path:
        file = new File(fileTextFields.get(
                epsToPDFpathIndex).getText());
        if (file.exists()) {
            properties.setProperty(
                    GraphicalUI.PROPERTY_EPS_TO_PDF_PATH,
                    fileTextFields.get(epsToPDFpathIndex)
                            .getText());
        } else {
            Object answer = GUIUtils
                    .questionDialog(
                            this,
                            Messages
                                    .formatMessage(
                                            INCORRECT_PATH_TO_FILE_QUESTION,
                                            file));
            if (answer.equals(JOptionPane.NO_OPTION)) return;
        }

        // Dot color theme:
        properties
                .setProperty(
                        GraphicalUI.PROPERTY_DOT_COLOR,
                        ((String) graphComboBoxes.get(
                                normalColorIndex)
                                .getSelectedItem())
                                .toUpperCase().replace(' ',
                                        '_'));
        properties.setProperty(
                GraphicalUI.PROPERTY_DOT_HIGHLIGHT_COLOR,
                ((String) graphComboBoxes.get(
                        highlightColorIndex)
                        .getSelectedItem()).toUpperCase()
                        .replace(' ', '_'));

        // entity types
        updateEntityTypes();

        // Apply UI's properties.
        owner.setProperties(properties);
        try {
            owner.applyProperties();
        } catch (BadPropertyValueException e) {
            logger.warning(e.getMessage());
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        setVisible(false);
        result = MAIN_OK_BUTTON_ACTION;
    }

    /**
     * <p>
     * This method initializes ActionListener for all
     * buttons.
     * </p>
     */
    private void createButtonListener() {
        buttonListener = new ActionListener() {

            /**
             * <p>
             * This method handles the action event and
             * calls the correspondive methods for each
             * action/button.
             * </p>
             */
            public void actionPerformed(ActionEvent e) {
                if (MAIN_OK_BUTTON_ACTION.equals(e
                        .getActionCommand())) {
                    mainOKButtonClicked();
                    return;
                }
                if (MAIN_CANCEL_BUTTON_ACTION.equals(e
                        .getActionCommand())) {
                    mainCancelButtonClicked();
                    return;
                }
                if (BROWSE_BUTTON_CLICKED.equals(e
                        .getActionCommand())) {
                    buttonBrowseClicked(e);
                    return;
                }
            }
        };
    }

    /**
     * <p>
     * Method for figuring out how the user exited the
     * dialog.
     * </p>
     * 
     * @return Returns either
     *         SettingsDialog.MAIN_OK_BUTTON_ACTION or
     *         SettingsDialog.MAIN_CANCEL_BUTTON_ACTION
     *         depending on how the user closed the dialog.
     */
    public String getResult() {
        return result;
    }

    /**
     * <p>
     * Static method to create and show SettingsDialog.
     * </p>
     * 
     * @param owner
     *            Owner of the dialog to be shown.
     */
    public static String showDialog(GraphicalUI owner) {

        // Create new SettingsDialog.
        SettingsDialog dialog = new SettingsDialog(owner);
        GUIUtils.centerDialog(dialog);

        // Show created dialog.
        dialog.setVisible(true);

        // Return dialog result.
        return dialog.getResult();
    }

}
