/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.dialog.entitytab;

import java.util.HashSet;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import ucot.Messages;
import ucot.model.AnalyzeModel;
import ucot.model.AnalyzeModelException;
import ucot.model.ModelEditor;

/**
 * <p>
 * This panel allows user to edit entitys attributes.
 * </p>
 */
public class AttributesPanel extends
        JTableEntityPropertiesTab {

    Logger logger = Logger.getLogger("ucot.gui");

    private static final long serialVersionUID = 1980835276224106020L;

    /**
     * Log message for informing creation of the new
     * attributs.
     */
    protected final static String NEW_ATTRIBUTES_LOG_MESSAGE = Messages
            .getString("EntityPropertiesDialog.NEW_ATTRIBUTES_LOG_MESSAGE");

    /**
     * Log message for informing deletion of removed
     * attributes.
     */
    protected final static String DELETED_ATTRIBUTES_LOG_MESSAGE = Messages
            .getString("EntityPropertiesDialog.DELETED_ATTRIBUTES_LOG_MESSAGE");

    /** Log message for informing change of attributes. */
    protected final static String CHANGED_ATTRIBUTES_LOG_MESSAGES = Messages
            .getString("EntityPropertiesDialog.CHANGED_ATTRIBUTES_LOG_MESSAGE");

    /** Header for the attributes tab. */
    protected final static String ATTRIBUTES_TAB_HEADER = Messages
            .getString("EntityPropertiesDialog.ATTRIBUTES_TAB_HEADER");

    // Button labels localized.
    /** Text for new attribute button. */
    protected static String NEW_ATTRIBUTE_BUTTON = Messages
            .getString("EntityPropertiesDialog.NEW_ATTRIBUTE_BUTTON");

    /** Text for delete attribute button. */
    protected static String DELETE_ATTRIBUTE_BUTTON = Messages
            .getString("EntityPropertiesDialog.DELETE_ATTRIBUTE_BUTTON");

    protected final static String NEW_ATTRIBUTE_BUTTON_ACTION = "NEW_ATTRIBUTE";

    protected final static String DELETE_ATTRIBUTE_BUTTON_ACTION = "DELETE_ATTRIBUTE";

    /** Headers for the attributes table. */
    protected final static String[] ATTRIBUTES_TABLE_COLUMNS = {
            // Header for attribute name column.
            Messages
                    .getString("EntityPropertiesDialog.ATTRIBUTE_COLUMN_NAME"),
            // Header for to cardinal column.
            Messages
                    .getString("EntityPropertiesDialog.ATTRIBUTE_TO_CARDINALITY_COLUMN_NAME"),
            // Header for from cardinal column.
            Messages
                    .getString("EntityPropertiesDialog.ATTRIBUTE_FROM_CARDINALITY_COLUMN_NAME") };

    /**
     * <p>
     * Creates the panel using the localized strings readed
     * from the <code>Messages</code> object.
     * </p>
     */
    public AttributesPanel() {
        super(ATTRIBUTES_TAB_HEADER,
                ATTRIBUTES_TABLE_COLUMNS);
        // Create required buttons.
        addButton(NEW_ATTRIBUTE_BUTTON,
                NEW_ATTRIBUTE_BUTTON_ACTION);
        addButton(DELETE_ATTRIBUTE_BUTTON,
                DELETE_ATTRIBUTE_BUTTON_ACTION);
    }

    /**
     * Action performed: User clicked the 'new attribute'
     * button.
     */
    private void newAttributeButtonClicked() {
        // Add new attribute relation to attributes table.
        Vector<String> row = new Vector<String>();
        row.add("");
        row.add("1");
        row.add("1");
        getModel().addRow(row);
    }

    /**
     * Action performed: User clicked the 'delete attribute'
     * button.
     */
    private void deleteAttributeButtonClicked() {
        // Stop cell editing.
        if (getTable().isEditing()) getTable()
                .getCellEditor().stopCellEditing();
        // Delete selected row from relations table.
        if (getTable().getSelectedRow() < 0) return;
        getModel().removeRow(getTable().getSelectedRow());
    }

    /**
     * This method is called when an action is performed and
     * it calls the corresponding methods to handle the
     * action.
     * 
     * @param cmd
     *            action command
     */
    public void action(String cmd) {
        if (NEW_ATTRIBUTE_BUTTON_ACTION.equals(cmd)) {
            newAttributeButtonClicked();
            return;
        }

        if (DELETE_ATTRIBUTE_BUTTON_ACTION.equals(cmd)) {
            deleteAttributeButtonClicked();
            return;
        }

    }

    @Override
    public void load(AnalyzeModel analyzeModel,
            String entityName) {
        // TODO Auto-generated method stub
        super.load(analyzeModel, entityName);
        updateCellEditor(analyzeModel, entityName, 0, true);
        Set<String> attributes = analyzeModel.getEditor()
                .getAttributeNames(entityName);

        for (String attribute : attributes) {
            Vector<String> row = new Vector<String>();
            if (attribute.equals(entityName)) {
                row.addElement(SELF_POINTER_NAME);
            } else {
                row.addElement(attribute);
            }
            try {
                row.addElement(analyzeModel.getEditor()
                        .getAttributeToCardinal(entityName,
                                attribute));
                row.addElement(analyzeModel.getEditor()
                        .getAttributeFromCardinal(
                                entityName, attribute));
                getModel().addRow(row);
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void save(AnalyzeModel model,
            String saveEntityName) {
        super.save(model, saveEntityName);
        ModelEditor editor = model.getEditor();
        Set<String> oldAttributes = editor
                .getAttributeNames(saveEntityName);
        Set<String> currentAttributes = new HashSet<String>();
        // Cardinalities are expressed here as a 3-element
        // String array, which
        // contains {attributeName, fromCardinal,
        // toCardinal}.
        Set<String[]> oldCardinalities = new HashSet<String[]>();
        Set<String[]> currentCardinalities = new HashSet<String[]>();

        // Initialize old attributes and cardinalities sets.
        for (String attribute : oldAttributes) {
            try {
                String[] cardinality = {
                        attribute,
                        editor.getAttributeFromCardinal(
                                saveEntityName, attribute),
                        editor.getAttributeToCardinal(
                                saveEntityName, attribute) };
                oldCardinalities.add(cardinality);
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
        }

        // Initialize current attributes and cardinalities
        // sets.
        for (int i = 0; i < getModel().getRowCount(); i++) {

            String attributeName = ((String) getModel()
                    .getValueAt(i, 0)).trim();
            if (attributeName.equals(SELF_POINTER_NAME)) {
                attributeName = saveEntityName;
            }
            String attributeToCardinality = ((String) getModel()
                    .getValueAt(i, 1)).trim();
            String attributeFromCardinality = ((String) getModel()
                    .getValueAt(i, 2)).trim();

            if (attributeName.length() > 0) {
                currentAttributes.add(attributeName);
                String[] cardinality = { attributeName,
                        attributeFromCardinality,
                        attributeToCardinality };
                currentCardinalities.add(cardinality);
            }
        }

        // Calculate new attributes and deleted attributes.
        Set<String> newAttributes = new HashSet<String>();
        newAttributes.addAll(currentAttributes);
        newAttributes.removeAll(oldAttributes);
        Set<String> deletedAttributes = new HashSet<String>();
        deletedAttributes.addAll(oldAttributes);
        deletedAttributes.removeAll(currentAttributes);
        if (newAttributes.size() > 0
                || deletedAttributes.size() > 0) {
            // owner.setChanged(true);
        }

        logger.fine(Messages.formatMessage(
                NEW_ATTRIBUTES_LOG_MESSAGE, newAttributes));
        logger.fine(Messages.formatMessage(
                DELETED_ATTRIBUTES_LOG_MESSAGE,
                deletedAttributes));

        // Add new attributes to the analyze model.
        for (String attributeName : newAttributes) {
            try {
                editor.addAttribute(saveEntityName,
                        attributeName);
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
        }

        // Remove non-existing attribute relationships from
        // the analyze model.
        for (String attributeName : deletedAttributes) {
            try {
                editor.removeAttribute(saveEntityName,
                        attributeName);
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
        }

        // Set cardinalities.
        Set<String> changedAttributes = new HashSet<String>();
        for (String attribute : currentAttributes) {
            String fromCardinality = "1", toCardinality = "1";
            for (String[] oldCardinality : oldCardinalities) {
                if (oldCardinality[0].equals(attribute)) {
                    fromCardinality = oldCardinality[1];
                    toCardinality = oldCardinality[2];
                    break;
                }
            }
            for (String[] newCardinality : currentCardinalities) {
                if (newCardinality[0].equals(attribute)) {
                    if (!fromCardinality
                            .equals(newCardinality[1])
                            || !toCardinality
                                    .equals(newCardinality[2])) {
                        changedAttributes.add(attribute);
                    }
                    fromCardinality = newCardinality[1];
                    toCardinality = newCardinality[2];
                    break;
                }
            }
            try {
                editor.setAttributeFromCardinal(
                        saveEntityName, attribute,
                        fromCardinality);
                editor.setAttributeToCardinal(
                        saveEntityName, attribute,
                        toCardinality);
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
        }

        if (changedAttributes.size() > 0) {
            // owner.setChanged(true);
        }

        logger.fine(Messages.formatMessage(
                CHANGED_ATTRIBUTES_LOG_MESSAGES,
                changedAttributes));
    }

}
