/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.dialog.entitytab;

import java.util.HashSet;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import ucot.Messages;
import ucot.model.AnalyzeModel;
import ucot.model.AnalyzeModelException;
import ucot.model.ModelEditor;
import ucot.model.NoSuchEntityException;

/**
 * <p>
 * This panel allows user to edit entitys childs.
 * </p>
 */
public class ChildrenPanel extends
        JTableEntityPropertiesTab {

    private static final long serialVersionUID = 4303304569865873821L;

    /** Log message for informing creation of new children. */
    protected final static String NEW_CHILDREN_LOG_MESSAGE = Messages
            .getString("EntityPropertiesDialog.NEW_CHILDREN_LOG_MESSAGE");

    /**
     * Log message for informing deletion of removed
     * children.
     */
    protected final static String DELETED_CHILDREN_LOG_MESSAGE = Messages
            .getString("EntityPropertiesDialog.DELETED_CHILDREN_LOG_MESSAGE");

    /** Text for new child button. */
    protected static String NEW_CHILD_BUTTON = Messages
            .getString("EntityPropertiesDialog.NEW_CHILD_BUTTON");

    /** Text for delete child button. */
    protected static String DELETE_CHILD_BUTTON = Messages
            .getString("EntityPropertiesDialog.DELETE_CHILD_BUTTON");

    protected final static String NEW_CHILD_BUTTON_ACTION = "NEW_CHILD";

    protected final static String DELETE_CHILD_BUTTON_ACTION = "DELETE_CHILD";

    /** Headers for the method table. */
    protected final static String[] CHILDREN_TABLE_COLUMNS = {
    // Header for method name column.
    Messages
            .getString("EntityPropertiesDialog.CHILD_COLUMN_NAME") };

    /** Header for the children tab. */
    protected static String CHILDREN_TAB_HEADER = Messages
            .getString("EntityPropertiesDialog.CHILDREN_TAB_HEADER");

    private final static Logger logger = Logger
            .getLogger("ucot.gui");

    /**
     * <p>
     * Creates the panel using the localized strings readed
     * from the <code>Messages</code> object.
     * </p>
     */
    public ChildrenPanel() {
        super(CHILDREN_TAB_HEADER, CHILDREN_TABLE_COLUMNS);
        // Create required buttons. -->
        addButton(NEW_CHILD_BUTTON, NEW_CHILD_BUTTON_ACTION);
        addButton(DELETE_CHILD_BUTTON,
                DELETE_CHILD_BUTTON_ACTION);
    }

    /**
     * Action performed: User clicked the 'new child'
     * button.
     */
    private void newParentButtonClicked() {
        // Add new child to the children table.
        Vector<String> row = new Vector<String>();
        row.add("");
        getModel().addRow(row);
    }

    /**
     * Action performed: User clicked the 'delete child'
     * button.
     */
    private void deleteParentButtonClicked() {
        // Stop cell editing.
        if (getTable().isEditing()) getTable()
                .getCellEditor().stopCellEditing();
        // Delete selected row from the children table.
        if (getTable().getSelectedRow() < 0) return;
        getModel().removeRow(getTable().getSelectedRow());
    }

    /**
     * This method is called when an action is performed and
     * it calls the corresponding methods to handle the
     * action.
     */
    public void action(String cmd) {
        if (NEW_CHILD_BUTTON_ACTION.equals(cmd)) {
            newParentButtonClicked();
            return;
        }

        if (DELETE_CHILD_BUTTON_ACTION.equals(cmd)) {
            deleteParentButtonClicked();
            return;
        }
    }

    // TODO comment me
    public void save(AnalyzeModel analyzeModel,
            String saveEntityName) {

        super.save(analyzeModel, saveEntityName);
        ModelEditor editor = analyzeModel.getEditor();
        Set<String> oldChildren = editor
                .getChildren(saveEntityName);
        Set<String> currentChildren = new HashSet<String>();

        // Initialize current children.
        for (int i = 0; i < tableModel.getRowCount(); i++) {
            String childName = ((String) tableModel
                    .getValueAt(i, 0)).trim();
            // Paretn cannot be zero length or the object
            // itself cant be its
            // child.
            if (childName.length() > 0
                    && !childName.equals(saveEntityName)) {
                currentChildren.add(childName);
            }
        }

        // Calculate new children and deleted children.
        Set<String> newChildren = new HashSet<String>();
        newChildren.addAll(currentChildren);
        newChildren.removeAll(oldChildren);
        Set<String> deletedChildren = new HashSet<String>();
        deletedChildren.addAll(oldChildren);
        deletedChildren.removeAll(currentChildren);
        if (newChildren.size() > 0
                || deletedChildren.size() > 0) {
            // owner.setChanged(true);
        }

        logger.fine(Messages.formatMessage(
                NEW_CHILDREN_LOG_MESSAGE, newChildren));
        logger.fine(Messages.formatMessage(
                DELETED_CHILDREN_LOG_MESSAGE,
                deletedChildren));

        Set<String> entityNames = editor.getEntityNames();
        // Add new children to the analyze model.
        for (String childName : newChildren) {
            if (!childName.equals(saveEntityName)) {
                if (entityNames.contains(childName)) {
                    try {
                        editor.addChild(saveEntityName,
                                childName);
                    } catch (NoSuchEntityException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        // Remove non existing child relationships from the
        // analyze model.
        for (String childName : deletedChildren) {
            try {
                editor.removeParent(saveEntityName,
                        childName);
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
        }

    }

    // TODO comment me
    public void load(AnalyzeModel analyzeModel,
            String loadEntityName) {
        super.load(analyzeModel, loadEntityName);
        updateCellEditor(analyzeModel, loadEntityName, 0,
                false);
        Set<String> children = analyzeModel.getEditor()
                .getChildren(loadEntityName);
        for (String child : children) {
            if (child.equals(loadEntityName)) child = SELF_POINTER_NAME;
            Vector<String> row = new Vector<String>();
            row.addElement(child);
            getModel().addRow(row);
        }
    }

}
