/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.dot;

/**
 * <p>
 * Class that implements a color model used in dot graph
 * drawing environment. This model can be used for both main
 * colors and highlights.
 * </p>
 * 
 * @author UCOT
 */
public class DotColorModel {

    /**
     * ColorModel sets available for using with Dot
     * 
     * @author tujupien
     * 
     */
    public static enum ColorModel {
        BLACK_ON_WHITE, WHITE_ON_BLACK, GREEN, BLUE, RED
    };

    private static final ColorModel DEFAULT_COLOR = ColorModel.BLACK_ON_WHITE;

    private String edgeColor = "";

    private String fillColor = "";

    private String fontColor = "";

    private ColorModel model;

    /**
     * <p>
     * Constructor which sets all the colors as specified.
     * </p>
     * 
     * @param edgeColor
     *            <code>Color</code> of edges in the
     *            graph.
     * @param fillColor
     *            <code>Color</code> for filling the
     *            elements on the graph. Can also be used
     *            for graph background.
     * @param fontColor
     *            Text color.
     */
    private DotColorModel(String edgeColor,
            String fillColor, String fontColor) {
        this.edgeColor = edgeColor;
        this.fillColor = fillColor;
        this.fontColor = fontColor;
    }

    /**
     * <p>
     * Constructor for quick color model initialization.
     * </p>
     * 
     * @param drawColor
     *            <code>Color</code> for all edges and
     *            text in the graph.
     * @param fillColor
     *            Background color for the whole graph and
     *            all elements.
     */
    private DotColorModel(String drawColor, String fillColor) {
        this(drawColor, fillColor, drawColor);
    }

    /**
     * <p>
     * Method for acquiring the edge color.
     * </p>
     * 
     * @return Edge color.
     */
    public String getEdgeColor() {
        return edgeColor;
    }

    /**
     * <p>
     * Method for acquiring the fill color.
     * </p>
     * 
     * @return Fill color.
     */
    public String getFillColor() {
        return fillColor;
    }

    /**
     * <p>
     * Method for acquiring the font color.
     * </p>
     * 
     * @return Font color.
     */
    public String getFontColor() {
        return fontColor;
    }

    /**
     * <p>
     * Method for acquiring the name of the current model.
     * </p>
     * 
     * @return Color model's name.
     */
    public String getModel() {
        return model.toString();
    }

    /**
     * <p>
     * This static method creates a blue dot color model.
     * </p>
     * 
     * @return Blue dot color model.
     */
    private static DotColorModel blue() {
        DotColorModel result = new DotColorModel(
                "dodgerblue", "lightblue");
        result.model = ColorModel.BLUE;
        return result;
    }

    /**
     * <p>
     * This static method creates a green dot color model.
     * </p>
     * 
     * @return Green dot color model.
     */
    private static DotColorModel green() {
        DotColorModel result = new DotColorModel(
                "forestgreen", "chartreuse");
        result.model = ColorModel.GREEN;
        return result;
    }

    /**
     * <p>
     * This static method creates a black text on white
     * background dot color model.
     * </p>
     * 
     * @return Black text on white background dot color
     *         model.
     */
    private static DotColorModel blackOnWhite() {
        DotColorModel result = new DotColorModel("black",
                "white");
        result.model = ColorModel.BLACK_ON_WHITE;
        return result;
    }

    /**
     * <p>
     * This static method creates a white text on black
     * background dot color model.
     * </p>
     * 
     * @return White text on black background dot color
     *         model.
     */
    private static DotColorModel whiteOnBlack() {
        DotColorModel result = new DotColorModel("white",
                "black");
        result.model = ColorModel.WHITE_ON_BLACK;
        return result;
    }

    /**
     * <p>
     * This static method creates a red dot color model.
     * </p>
     * 
     * @return Red dot color model.
     */
    private static DotColorModel red() {
        DotColorModel result = new DotColorModel(
                "firebrick", "tomato");
        result.model = ColorModel.RED;
        return result;
    }

    // TODO Add more color models.

    /**
     * <p>
     * Static method for creating a color model from
     * available sets.
     * </p>
     */
    public static DotColorModel color(ColorModel colorModel) {
        switch (colorModel) {
        case BLUE:
            return blue();
        case GREEN:
            return green();
        case BLACK_ON_WHITE:
            return blackOnWhite();
        case WHITE_ON_BLACK:
            return whiteOnBlack();
        case RED:
            return red();
        default:
            return blackOnWhite();
        }
    }

    /**
     * <p>
     * Static method for creating a color model from
     * available sets.
     * </p>
     */
    public static DotColorModel color(String colorModel) {
        try {
            return color(ColorModel.valueOf(colorModel));
        } catch (IllegalArgumentException e) {
            return color(DEFAULT_COLOR);
        }
    }
}
