/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.tree.analyzemodeltree;

import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Vector;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreePath;

import ucot.model.AnalyzeModel;
import ucot.model.Updation;
import ucot.ui.gui.GraphicalUI;

/**
 * <p>
 * This is the <code>TreeModel</code> for
 * <code>JTree</code> representing the
 * <code>AnalyzeModel</code>. It is able to listen the
 * <code>Udation</code> messages coming from the
 * <code>AnalyzeModel</code> and its ask the tree to
 * update itself when the model is updated.
 * </p>
 * <p>
 * This tree and the <code>TreeItem</code> classes it uses
 * are actually a adapter for the analyze model because the
 * <code>TreeItems</code> are created dynamically based on
 * the status of the actual model.
 * </p>
 * <p>
 * <h4>Known problems</h4>
 * <p>
 * Because of the dynamic nature of this tree there are
 * problems informing the actual tree about the
 * modifications and currently the tree is invaliated
 * completely after every minor change. Modify
 * <code>Update(Observer,Object)</code>-method to to fix
 * this.
 * </p>
 * 
 * @author pajumasu
 * @see ucot.model.AnalyzeModel
 * @see javax.swing.JTree
 * @see javax.swing.tree.TreeModel
 * @see ucot.ui.gui.tree.analyzemodeltree.TreeItem
 */
public class AnalyzeTreeModel implements
        javax.swing.tree.TreeModel, Observer {
    private GraphicalUI owner;

    AnalyzeModel model;

    Set<TreeModelListener> listeners = new HashSet<TreeModelListener>();

    TreeItem root;

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable,
     *      java.lang.Object)
     */
    public void update(Observable o, Object arg) {
        if (!(arg instanceof Updation)) return;
        Updation updation = (Updation) arg;

        List<TreeItem> relevants = new Vector<TreeItem>();

        List<List<TreeItem>> paths = new Vector<List<TreeItem>>();

        for (TreeItem item : relevants) {
            List<TreeItem> path = item.getPath();
            if (path != null && path.size() > 0) {
                paths.add(path);
            }
        }

        for (List<TreeItem> path : paths) {
            System.out.println("Refreshing path: " + path);
            if (path.size() > 0) {
                switch (updation.getType()) {
                case NEW:
                    sendTreeStructureChanged(new TreeModelEvent(
                            this, path.toArray()));
                    break;
                case DELETION:
                    sendTreeStructureChanged(new TreeModelEvent(
                            this, new Object[] { root }));
                    break;
                case MODIFY:
                    sendTreeNodesChanged(new TreeModelEvent(
                            this, path.toArray()));
                }
            }
        }
        // Hack. Because we reseive notify after items have
        // been deleted there
        // might
        // not be path to the wanted item. So we need to
        // refresh whole tree.
        if (paths.size() == 0) {
            sendTreeStructureChanged(new TreeModelEvent(
                    this, new Object[] { root }));
        }
    }

    /**
     * Returns the analyze model which this item models.
     * 
     * @return the analyze model which this item models.
     */
    public AnalyzeModel getAnalyzeModel() {
        return model;
    }

    /**
     * <p>
     * Creates the object.
     * </p>
     * 
     * @param owner
     *            The owner of this component.
     */
    public AnalyzeTreeModel(GraphicalUI owner) {
        this.owner = owner;
        updateAnalyzeModel();
    }

    /**
     * <p>
     * Updates the <code>AnalyzeModel</code> from the
     * <code>Core</code> component. This should be called
     * if the
     * <code>AnalyzeModel</core> in the <code>Core</code>
     * is changed to other object.
     * </p>
     */
    public void updateAnalyzeModel() {
        model = owner.getControlInterface()
                .getAnalyzeModel();
        root = new EntitiesTreeItem(this, null);
        sendTreeStructureChanged(new TreeModelEvent(this,
                new Object[] { root }));
        model.addObserver(this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#getRoot()
     */
    public Object getRoot() {
        return root;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#getChild(java.lang.Object,
     *      int)
     */
    public Object getChild(Object parent, int index) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            Object ret = null;
            if (children != null) {
                if (index < children.size()) {
                    ret = children.get(index);
                }
            }
            return ret;
        } else {
            throw new UnsupportedOperationException();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#getChildCount(java.lang.Object)
     */
    public int getChildCount(Object parent) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            int ret = 0;
            if (children != null) {
                ret = children.size();
            }
            return ret;
        } else {
            throw new UnsupportedOperationException();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#isLeaf(java.lang.Object)
     */
    public boolean isLeaf(Object node) {
        return getChildCount(node) == 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#valueForPathChanged(javax.swing.tree.TreePath,
     *      java.lang.Object)
     */
    public void valueForPathChanged(TreePath path,
            Object newValue) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#getIndexOfChild(java.lang.Object,
     *      java.lang.Object)
     */
    public int getIndexOfChild(Object parent, Object child) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            int index = -1;
            if (null != children) {
                index = children.indexOf(child);
            }
            return index;
        } else {
            throw new UnsupportedOperationException();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#addTreeModelListener(javax.swing.event.TreeModelListener)
     */
    public void addTreeModelListener(TreeModelListener l) {
        listeners.add(l);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#removeTreeModelListener(javax.swing.event.TreeModelListener)
     */
    public void removeTreeModelListener(TreeModelListener l) {
        listeners.remove(l);

    }

    /**
     * <p>
     * Sends <code>treeNodesChanged</code> event to all
     * <code>TreeModelListeners</code> listening this
     * object.
     * </p>
     * 
     * @param event
     *            The actual event to be sended.
     * @see TreeModelListener#treeNodesChanged(javax.swing.event.TreeModelEvent)
     */
    protected void sendTreeNodesChanged(TreeModelEvent event) {
        for (TreeModelListener listener : listeners) {
            listener.treeNodesChanged(event);
        }
    }

    /**
     * <p>
     * Sends <code>treeNodesInserted</code> event to all
     * <code>TreeModelListeners</code> listening this
     * object.
     * </p>
     * 
     * @param event
     *            The actual event to be sended.
     * @see TreeModelListener#treeNodesInserted(javax.swing.event.TreeModelEvent)
     */
    protected void sendTreeNodesInserted(
            TreeModelEvent event) {
        for (TreeModelListener listener : listeners) {
            listener.treeNodesInserted(event);
        }
    }

    /**
     * <p>
     * Sends <code>treeNodesRemoved</code> event to all
     * <code>TreeModelListeners</code> listening this
     * object.
     * </p>
     * 
     * @param event
     *            The actual event to be sended.
     * @see TreeModelListener#treeNodesRemoved(javax.swing.event.TreeModelEvent)
     */
    protected void sendTreeNodesRemoved(TreeModelEvent event) {
        for (TreeModelListener listener : listeners) {
            listener.treeNodesRemoved(event);
        }
    }

    /**
     * <p>
     * Sends <code>treeStructureChanged</code> event to
     * all <code>TreeModelListeners</code> listening this
     * object.
     * </p>
     * 
     * @param event
     *            The actual event to be sended.
     * @see TreeModelListener#treeStructureChanged(javax.swing.event.TreeModelEvent)
     */
    protected void sendTreeStructureChanged(
            TreeModelEvent event) {
        for (TreeModelListener listener : listeners) {
            listener.treeStructureChanged(event);
        }
    }

}
