/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.tree.usecasetree;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;
import java.util.List;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import ucot.Messages;
import ucot.core.ControlInterface;
import ucot.heuristic.HeuristicInterface;
import ucot.input.UseCase;
import ucot.parser.ParserInterface;
import ucot.ui.UseCasePanelInterface;
import ucot.ui.gui.dialog.AddToModelWithDialog;
import ucot.ui.gui.dot.DotPanel;

/**
 * <p>
 * <code>JTree</code> for displaying usecases that are
 * loaded into program.
 * <p>
 * 
 * <p>
 * The data model of this tree is kept in
 * <code>UseCaseTreeModel</code>.
 * </p>
 * 
 * <p>
 * Root node of this tree is <code>FilesTreeItem</code>.
 * Root has all <code>UseCase</code> sources loaded into
 * program as children as <code>FileTreeItem</code>s.
 * <code>FileTreeItem</code> holds <code>UseCase</code>s
 * from that source as it's child nodes as
 * <code>UseCaseTreeItem</code>s.
 * </p>
 * 
 * @see JTree
 * 
 * @author ilanliuk
 * 
 */
public class UseCaseTree extends JTree {

    private static final long serialVersionUID = -6655430305570481199L;

    private ActionListener popupMenuActionListener;

    private UseCaseTreeModel useCaseTreeModel;

    private DotPanel dotPanel = null;

    private ControlInterface core = null;

    private UseCasePanelInterface useCasePanel;

    //
    // Icons for UseCaseTrees TreeItems.
    protected static Icon usecaseIcon = new ImageIcon(
            "img/icon/usecase_icon.gif"); //$NON-NLS-1$

    protected static Icon usecaseLeafIcon = new ImageIcon(
            "img/icon/usecase_leaf_icon.gif"); //$NON-NLS-1$

    protected static Icon usecaseIconNotInModel = new ImageIcon(
            "img/icon/usecase_not_in_model_icon.gif"); //$NON-NLS-1$

    protected static Icon usecaseLeafIconNotInModel = new ImageIcon(
            "img/icon/usecase_leaf_not_in_model_icon.gif"); //$NON-NLS-1$

    //
    // ActionCommand-strings for popup-menu actions
    private static final String ADD_SINGLE_POPUP_ACTION = "ADD_SINGLE"; //$NON-NLS-1$

    private static final String ADD_SINGLE_WITH_POPUP_ACTION = "ADD_SINGLE_WITH"; //$NON-NLS-1$

    private static final String ADD_SUBTREE_POPUP_ACTION = "ADD_SUBTREE"; //$NON-NLS-1$

    private static final String ADD_SUBTREE_WITH_POPUP_ACTION = "ADD_SUBTREE_WITH"; //$NON-NLS-1$

    /**
     * <p>
     * Custom <code>TreeCellRenderer</code> for
     * highlighting tree nodes and adding custom icons for
     * them.
     * </p>
     * 
     */
    private class UseCaseTreeCellRenderer extends
            DefaultTreeCellRenderer {

        private static final long serialVersionUID = -3044905713499088318L;

        // fonts that are used in drawing UseCaseTreeItems
        // highlightFont is for items that are already in
        // analyzemodel.
        private Font defaultFont = new Font(
                "Dialog", Font.PLAIN, 12); //$NON-NLS-1$

        private Font highlightFont = new Font(
                "Dialog", Font.ITALIC, 12); //$NON-NLS-1$

        @Override
        public Component getTreeCellRendererComponent(
                JTree tree, Object value, boolean sel,
                boolean expanded, boolean leaf, int row,
                boolean hasFocus) {

            super.getTreeCellRendererComponent(tree, value,
                    sel, expanded, leaf, row, hasFocus);

            setFont(defaultFont);

            // Get Icon for this treeitem
            if (value instanceof TreeItem) {
                TreeItem item = (TreeItem) value;
                setIcon(item.getIcon());
            }

            if (value instanceof UseCaseTreeItem) {
                UseCaseTreeItem item = (UseCaseTreeItem) value;

                // check if need to use highlight
                // on this tree item.
                if (item.isUsecaseInAnalyzemodel()) {
                    setFont(highlightFont);

                }
            }
            return this;
        }
    }

    /**
     * <p>
     * Default constructor for <code>UseCaseTree</code>.
     * </p>
     * 
     * @param core
     *            <code>ControlInterface</code> which
     *            holds the <code>UseCaseCollection</code>
     *            which contains the <code>UseCase</code>s
     *            we want to show.
     * 
     * 
     */
    public UseCaseTree(ControlInterface core) {
        super();
        this.core = core;
        this.useCaseTreeModel = new UseCaseTreeModel(
                this.core.getUseCaseCollection());
        this.getSelectionModel().setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);

        this.setModel(this.useCaseTreeModel);

        // Create custom CellRenderer so we can highlight
        // the items
        // we want and initialize icons for it.
        this.setCellRenderer(new UseCaseTreeCellRenderer());

        // create and add some listeners for
        // the tree and popup-menu..
        this
                .addTreeSelectionListener(getTreeSelectionListener());
        this.addMouseListener(getMouseListener());
        this.popupMenuActionListener = getActionListenerForPopupMenu();
    }

    /**
     * <p>
     * Method to add <code>UseCase</code> pointed at
     * <code>UseCaseTreeItem</code> and it's sub use cases
     * into <code>Vector</code>.
     * </p>
     * 
     * @param item
     *            <code>UseCaseTreeItem</code> to add into
     *            <code>Vector</code>.
     * @return <code>Vector</code> containing
     *         <code>UseCase</code>s.
     */
    @SuppressWarnings(value = { "unchecked" } // Suppreess
                                                // type
                                                // safty
    // warnings.
    )
    private Vector<UseCase> getSubtreeInVector(
            UseCaseTreeItem item) {

        // create vector for usecases
        Vector<UseCase> result = new Vector<UseCase>();

        // add subusecases recursively to vector
        List<UseCaseTreeItem> list = item.getChildren();
        for (UseCaseTreeItem child : list) {
            result.addAll(getSubtreeInVector(child));
        }

        // if usecase is not already
        // in model, add it
        if (!item.getUseCase().isUseCaseAnalyzed()) {
            result.add(item.getUseCase());
        }

        // return vector of UseCases
        // from this subtree
        return result;
    }

    /**
     * <p>
     * Method to add all <code>UseCase</code>s from file
     * pointed out at <code>FileTreeItem</code> into
     * <code>Vector</code>.
     * </p>
     * 
     * @param item
     *            <code>FileTreeItem</code> which contains
     *            the source from which we want to get the
     *            use cases.
     * @return <code>Vector</code> containing
     *         <code>UseCase</code>s from given source.
     */
    private Vector<UseCase> getSubtreeInVector(
            FileTreeItem item) {

        // create vector for usecases
        Vector<UseCase> result = new Vector<UseCase>();

        // get usecases from this file
        List<TreeItem> list = item.getChildren();

        // add usecases to vector
        for (TreeItem child : list) {
            UseCaseTreeItem useCaseTreeItem = (UseCaseTreeItem) child;
            result
                    .addAll(getSubtreeInVector(useCaseTreeItem));
        }

        // return usecases
        return result;
    }

    /**
     * <p>
     * Method to add <code>UseCase</code>s from subtree
     * of this <code>TreeItem</code> into
     * <code>Vector</code>.
     * </p>
     * 
     * @param item
     *            <code>TreeItem<code> which sub use cases we want to get.
     * @return <code>Vector</code> containing <code>UseCase</code>s.
     */
    private Vector<UseCase> getSubtreeInVector(TreeItem item) {

        // Check what kind of treeitem we just got..
        if (item instanceof FilesTreeItem) { return getSubtreeInVector((FilesTreeItem) item); }

        if (item instanceof FileTreeItem) { return getSubtreeInVector((FileTreeItem) item); }

        if (item instanceof UseCaseTreeItem) { return getSubtreeInVector((UseCaseTreeItem) item); }

        // shouldn't get here, but return null anyway
        return null;
    }

    /**
     * <p>
     * Method to add all use case sources that are loaded
     * into program into <code>Vector</code>.
     * </p>
     * 
     * @param item
     *            <code>FilesTreeItem</code>
     * @return <code>Vector</code> containing
     *         <code>UseCase</code>s.
     */
    @SuppressWarnings(value = { "unchecked" } // Suprreess
                                                // type
                                                // safty
    // warnings.
    )
    private Vector<UseCase> getSubtreeInVector(
            FilesTreeItem item) {

        // create vector for usecases
        Vector<UseCase> result = new Vector<UseCase>();

        // get all loaded usecase-files
        List<FileTreeItem> list = item.getChildren();

        // add files to model
        for (FileTreeItem file : list) {
            result.addAll(getSubtreeInVector(file));
        }

        // return usecases
        return result;
    }

    /**
     * <p>
     * Adds single
     * <code>UseCase<code> into analyzemodel with given 
     * <code>ParserInterface</code> and <code>HeuristicInterface</code>.<p>
     * 
     * @param usecase <code>UseCase</code> to add into model.
     * @param parser <code>ParserInterface</code> to use for parsing <code>UseCase</code>.
     * @param heuristic <code>HeuristicInterface</code> to use on the use case.
     */
    private void addToAnalyzeModel(UseCase usecase,
            ParserInterface parser,
            HeuristicInterface heuristic) {

        // Check if usecase is already in model
        // if it is we do nothing.
        if (usecase.isUseCaseAnalyzed()) { return; }

        // create vector for usecase and add it into vector
        Vector<UseCase> usecases = new Vector<UseCase>();
        usecases.add(usecase);

        // try to add usecase into model
        try {
            this.core.addToAnalyzeModel(usecases);
        } catch (Exception ex) {
            System.err
                    .println(Messages
                            .getFormatedMessage(
                                    "UseCaseTree.ERROR_WHEN_ADDING_USECASE_INTO_MODEL", usecase.getName())); //$NON-NLS-1$
            ex.printStackTrace();
        }
    }

    /**
     * <p>
     * Adds single
     * <code>UseCase<code> into analyzemodel with default 
     * <code>ParserInterface</code> and <code>HeuristicInterface</code>.<p>
     * 
     * @param usecase <code>UseCase</code> to add into model.
     */
    private void addToAnalyzeModel(UseCase usecase) {
        addToAnalyzeModel(usecase, this.core
                .getCurrentParser(), this.core
                .getCurrentHeuristic());
    }

    /**
     * <p>
     * Add <code>Vector</code> of use cases into
     * analyzemodel with given <code>ParserInterface</code>
     * and <code>HeuristicInterface</code>.
     * </p>
     * 
     * @param usecases
     *            <code>Vector</code> of
     *            <code>UseCase</code>s to add.
     * @param parser
     *            <code>ParserInterface</code> to use.
     * @param heuristic
     *            <code>HeuristicInterface</code> to use.
     */
    private void addToAnalyzeModel(
            Vector<UseCase> usecases,
            ParserInterface parser,
            HeuristicInterface heuristic) {

        try {
            this.core.addToAnalyzeModel(usecases, parser,
                    heuristic);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    /**
     * <p>
     * Add <code>Vector</code> of use cases into
     * analyzemodel with default
     * <code>ParserInterface</code> and
     * <code>HeuristicInterface</code>.
     * </p>
     * 
     * @param usecases
     *            <code>Vector</code> of
     *            <code>UseCase</code>s to add.
     */
    private void addToAnalyzeModel(Vector<UseCase> usecases) {
        addToAnalyzeModel(usecases, this.core
                .getCurrentParser(), this.core
                .getCurrentHeuristic());
    }

    /**
     * <p>
     * Method to ask from user which
     * <code>ParserInterface</code> and
     * <code>HeuristicInterface</code> should be used to
     * add use case into model. Uses
     * <code>AddToModelWithDialog</code> to ask which the
     * wanted <code>ParserInterface</code> and
     * <code>HeuristicInterface</code> are.
     * </p>
     * 
     * @param usecase
     *            <code>UseCase</code> to add into model.
     * 
     * 
     */
    private void addToAnalyzeModelWith(UseCase usecase) {
        Vector<UseCase> usecases = new Vector<UseCase>();
        usecases.add(usecase);
        addToAnalyzeModelWith(usecases);
    }

    /**
     * <p>
     * Method to ask from user which
     * <code>ParserInterface</code> and
     * <code>HeuristicInterface</code> should be used to
     * add usecases into model. Uses
     * <code>AddToModelWithDialog</code> to ask which the
     * wanted <code>ParserInterface</code> and
     * <code>HeuristicInterface</code> are.
     * </p>
     * 
     * @param usecases
     *            <code>Vector</code> of
     *            <code>UseCase</code>s to add.
     * 
     * 
     */
    private void addToAnalyzeModelWith(
            Vector<UseCase> usecases) {

        // if showDialog returns false, user clicked Cancel
        // in dialog and we do nothing
        if (!AddToModelWithDialog
                .showDialog(UseCaseTree.this.core)) { return; }

        ParserInterface parser = AddToModelWithDialog
                .getParser();
        HeuristicInterface heuristic = AddToModelWithDialog
                .getHeuristic();

        // add usecases into model with choosed
        // parser and heuristics.
        addToAnalyzeModel(usecases, parser, heuristic);
    }

    /**
     * <p>
     * Creates and returns
     * <code>TreeSelectionListener</code> for
     * <code>UseCaseTree</code>.
     * </p>
     * 
     * @return <code>TreeSelectionListener</code>
     */
    private TreeSelectionListener getTreeSelectionListener() {

        // if UseCaseTreeItem -node is selected from tree
        // - highlight entities and relations in DotPanel
        // from that usecase
        // - show usecase steps in UsecasePanel
        //
        TreeSelectionListener listener = new TreeSelectionListener() {

            public void valueChanged(TreeSelectionEvent e) {

                TreeItem ti = (TreeItem) e.getPath()
                        .getLastPathComponent();

                if (ti instanceof UseCaseTreeItem) {

                    UseCaseTreeItem c = (UseCaseTreeItem) ti;

                    if (c.isUsecaseInAnalyzemodel()) {

                        // if use case is in analyze model,
                        // tell dotpanel
                        // to highlight entities from this
                        // use case.
                        if (dotPanel != null) {
                            dotPanel.highlight(c
                                    .getUseCase()
                                    .getAnalyzeModel());
                        }
                    } else {
                        dotPanel.highlight(null);
                    }
                    // Show selected usecase in useCasePanel
                    useCasePanel
                            .showUseCase(c.getUseCase());
                } else {
                    useCasePanel.Clear();
                }
            }

        };
        return listener;
    }

    /**
     * <p>
     * Creates and returns <code>MouseListener</code> for
     * <code>UseCaseTree</code>.
     * </p>
     * 
     * @return <code>MouseListener</code>
     */
    private MouseListener getMouseListener() {

        // Create custom MouseAdapter instead of
        // MouseListener
        // so we don't have to implement all MouseEvents.
        MouseListener mouselistener = new MouseAdapter() {

            // override mouseClicked event
            public void mouseClicked(MouseEvent e) {

                // get TreePath to node which was selected.
                final TreePath selectionPath = getPathForLocation(
                        e.getX(), e.getY());

                // if nothing was selected, just exit
                if (selectionPath == null) { return; }

                // set selection correct
                setSelectionPath(selectionPath);

                // if doubleclicked with first mousebutton
                // on UseCaseTreeItem add UseCase to model
                if (e.getButton() == MouseEvent.BUTTON1
                        && e.getClickCount() == 2) {

                    Object o = selectionPath
                            .getLastPathComponent();

                    if (o instanceof UseCaseTreeItem) {
                        UseCaseTreeItem item = (UseCaseTreeItem) o;

                        // try to add usecase into model
                        UseCaseTree.this
                                .addToAnalyzeModel(item
                                        .getUseCase());
                        repaint();
                        useCasePanel.refresh();
                    }
                }

                // if user clicked second mouse button
                // (MouseEvent.BUTTON2 is scroll)
                if (e.getButton() == MouseEvent.BUTTON3) {

                    // if selection was not TreeItem just
                    // exit..
                    final Object o = selectionPath
                            .getLastPathComponent();
                    if (!(o instanceof TreeItem)) { return; }

                    TreeItem item = (TreeItem) o;

                    // create popupmenu and options for it.
                    JPopupMenu popup = getPopupMenuFor(item);

                    // Show popup menu relative to the
                    // components
                    // coordinates.
                    // This shows popup exactly where user
                    // pressed.
                    popup.show(e.getComponent(), e.getX(),
                            e.getY());

                }
            }
        };

        // return the mouselistener we just created
        return mouselistener;
    }

    /**
     * <p>
     * Creates and returns <code>JPopupMenu</code> for
     * given <code>TreeItem</code>.
     * </p>
     * 
     * <p>
     * MenuItems in popup menu are depending of the type of
     * <code>TreeItem</code> we need the menu for.
     * </p>
     * 
     * @param treeitem
     *            <code>TreeItem</code> we need a popup
     *            menu for.
     * @return <code>JPopupMenu</code>
     */
    private JPopupMenu getPopupMenuFor(TreeItem treeitem) {

        /**
         * Custom JMenuItem class to change default Font of
         * JMenuItem.
         * 
         * @author ilanliuk
         * 
         */
        class CustomMenuItem extends JMenuItem {
            private static final long serialVersionUID = 5522473017661834097L;

            public CustomMenuItem(String text) {
                super(text);
                this.setFont(new Font(
                        "Dialog", Font.PLAIN, 12)); //$NON-NLS-1$
            }

        }

        JPopupMenu menu = new JPopupMenu();

        //
        // Create and initialize needed MenuItems
        JMenuItem addSingle = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_TO_MODEL_WITH_DEFAULTS")); //$NON-NLS-1$
        addSingle.setActionCommand(ADD_SINGLE_POPUP_ACTION);
        addSingle
                .addActionListener(popupMenuActionListener);

        JMenuItem addSingleWith = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_TO_MODEL_WITH")); //$NON-NLS-1$
        addSingleWith
                .setActionCommand(ADD_SINGLE_WITH_POPUP_ACTION);
        addSingleWith
                .addActionListener(popupMenuActionListener);

        JMenuItem addSubtree = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_SUBTREE_TO_MODEL_WITH_DEFAULTS")); //$NON-NLS-1$
        addSubtree
                .setActionCommand(ADD_SUBTREE_POPUP_ACTION);
        addSubtree
                .addActionListener(popupMenuActionListener);

        JMenuItem addSubtreeWith = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_SUBTREE_TO_MODEL_WITH")); //$NON-NLS-1$
        addSubtreeWith
                .setActionCommand(ADD_SUBTREE_WITH_POPUP_ACTION);
        addSubtreeWith
                .addActionListener(popupMenuActionListener);

        JMenuItem addFile = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_FILE_WITH_DEFAULTS")); //$NON-NLS-1$
        addFile.setActionCommand(ADD_SUBTREE_POPUP_ACTION);
        addFile.addActionListener(popupMenuActionListener);

        JMenuItem addFileWith = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_FILE_WITH")); //$NON-NLS-1$
        addFileWith
                .setActionCommand(ADD_SUBTREE_WITH_POPUP_ACTION);
        addFileWith
                .addActionListener(popupMenuActionListener);

        JMenuItem addFiles = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_FILES_WITH_DEFAULTS")); //$NON-NLS-1$
        addFiles.setActionCommand(ADD_SUBTREE_POPUP_ACTION);
        addFiles.addActionListener(popupMenuActionListener);

        JMenuItem addFilesWith = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_ADD_FILES_WITH")); //$NON-NLS-1$
        addFilesWith
                .setActionCommand(ADD_SUBTREE_WITH_POPUP_ACTION);
        addFilesWith
                .addActionListener(popupMenuActionListener);

        // Refresh Files menu items
        JMenuItem refreshFiles = new CustomMenuItem(
                Messages
                        .getString("UseCaseTree.MENU_REFRESH_FILES")); //$NON-NLS-1$
        refreshFiles
                .addActionListener(getRefreshFileActionListener());

        JMenuItem refreshFile = new CustomMenuItem(Messages
                .getString("UseCaseTree.MENU_REFRESH_FILE")); //$NON-NLS-1$
        refreshFile
                .addActionListener(getRefreshFileActionListener());
        // MenuItems
        //

        //
        // Add needed MenuItems into PopupMenu

        //
        // if selected is a UseCaseTreeItem
        // add option to menu to add it to model
        // with default parser and heuristics and
        // with selected parser and heuristics
        if (treeitem instanceof UseCaseTreeItem) {

            UseCaseTreeItem item = (UseCaseTreeItem) treeitem;

            // if the usecase this UseCaseTreeItem
            // contains is already in model
            // we don't add these options
            if (!item.isUsecaseInAnalyzemodel()) {
                menu.add(addSingle);
                menu.add(addSingleWith);
            }

            //
            // if UseCaseTreeItem has children, add also
            // option
            // to add whole subtree into model.
            if (!item.isLeaf()) {

                if (!item.isUsecaseInAnalyzemodel()) {
                    menu.addSeparator();
                }

                menu.add(addSubtree);
                menu.add(addSubtreeWith);
            }
        }

        //
        // if selected node is FileTreeItem add option to
        // menu
        // to add all UseCases from file into model
        if (treeitem instanceof FileTreeItem) {
            menu.add(addFile);
            menu.add(addFileWith);
            menu.addSeparator();
            menu.add(refreshFile);
        }

        //
        // if selected node is FileTreeItem add option to
        // menu
        // to add all UseCases from all files into model
        if (treeitem instanceof FilesTreeItem) {
            menu.add(addFiles);
            menu.add(addFilesWith);
            menu.addSeparator();
            menu.add(refreshFiles);
        }

        // return PopupMenu for given TreeItem
        return menu;
    }

    /**
     * <p>
     * Returns <code>ActionListener</code> for refreshing
     * files action from <code>UseCaseTree</code> popup
     * menu.
     * </p>
     * 
     * @return <code>ActionListener</code>
     */
    private ActionListener getRefreshFileActionListener() {

        ActionListener listener = new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {

                // get selected object from tree
                final TreePath selectionPath = UseCaseTree.this
                        .getSelectionPath();
                Object source = selectionPath
                        .getLastPathComponent();

                if (source instanceof FileTreeItem) {

                    FileTreeItem item = (FileTreeItem) source;
                    URL url = item.getUrl();

                    // reload the file from url
                    item.getUseCaseCollection()
                            .removeFromUrl(url);
                    core.loadUseCases(url);

                }

                if (source instanceof FilesTreeItem) {

                    FilesTreeItem item = (FilesTreeItem) source;
                    List childs = item.getChildren();

                    for (Object o : childs) {
                        if (o instanceof FileTreeItem) {

                            // get url from FileTreeItem
                            URL url = ((FileTreeItem) o)
                                    .getUrl();

                            // reload the file from url
                            item.getUseCaseCollection()
                                    .removeFromUrl(url);
                            core.loadUseCases(url);
                        }
                    }
                }
            }

        };
        return listener;
    }

    /**
     * Return <code>ActionListener</code> for
     * <code>UseCaseTree</code>s popup menu
     * 
     * @return <code>ActionListener</code> for popup menu
     */
    private ActionListener getActionListenerForPopupMenu() {

        ActionListener listener = new ActionListener() {

            public void actionPerformed(ActionEvent e) {

                // get selected object from tree
                final TreePath selectionPath = UseCaseTree.this
                        .getSelectionPath();
                Object source = selectionPath
                        .getLastPathComponent();

                // check what user wanted to do
                if (source instanceof UseCaseTreeItem) {
                    UseCaseTreeItem item = (UseCaseTreeItem) source;

                    if (e.getActionCommand().equals(
                            ADD_SINGLE_POPUP_ACTION)) {
                        addToAnalyzeModel(item.getUseCase());
                    }

                    if (e.getActionCommand().equals(
                            ADD_SINGLE_WITH_POPUP_ACTION)) {
                        addToAnalyzeModelWith(item
                                .getUseCase());
                    }
                }

                if (source instanceof TreeItem) {
                    TreeItem item = (TreeItem) source;

                    if (e.getActionCommand().equals(
                            ADD_SUBTREE_POPUP_ACTION)) {
                        addToAnalyzeModel(getSubtreeInVector(item));
                    }

                    if (e.getActionCommand().equals(
                            ADD_SUBTREE_WITH_POPUP_ACTION)) {
                        addToAnalyzeModelWith(getSubtreeInVector(item));
                    }
                }

                // force repaint
                UseCaseTree.this.repaint();
            }

        };
        return listener;
    }

    /**
     * Method to set the <code>DotPanel</code> which is
     * used to display the model.
     * 
     * @param panel
     *            <code>DotPanel</code>
     */
    public void setDotPanel(DotPanel panel) {
        dotPanel = panel;
    }

    /**
     * <p>
     * Method to set the <code>UseCasePanelInterface</code>
     * which is used to display steps of the selected use
     * case.
     * </p>
     * 
     * @param useCasePanel
     *            <code>UseCasePanelInterface</code>.
     * 
     * 
     * 
     */
    public void setUseCasePanel(
            UseCasePanelInterface useCasePanel) {
        this.useCasePanel = useCasePanel;
    }

    /**
     * <p>
     * Clears contest of this tree.
     * </p>
     * 
     */
    public void clear() {
        useCaseTreeModel.clear();
    }
}
