/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.tree.usecasetree;

import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import ucot.input.UseCaseCollection;

/**
 * <p>
 * Implementation of <code>TreeModel</code> for holding
 * data model of <code>UseCaseTree</code>.
 * </p>
 * 
 * 
 * 
 * 
 * @author ilanliuk
 * 
 */
public class UseCaseTreeModel implements TreeModel,
        Observer {

    protected Set<TreeModelListener> listeners = new HashSet<TreeModelListener>();

    private UseCaseCollection collection;

    private TreeItem root;

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable,
     *      java.lang.Object)
     */
    public void update(Observable o, Object arg) {
        for (TreeModelListener l : listeners) {
            l.treeStructureChanged(new TreeModelEvent(this,
                    new Object[] { root }));
        }
    }

    /**
     * <p>
     * Default constructor for <code>UseCaseTreeModel</code>.
     * </p>
     * 
     * <p>
     * Adds given <code>UseCaseCollection</code> observed
     * to get notified when data of the
     * <code>UseCaseCollection</code> changes.
     * </p>
     * 
     * @param u
     *            <code>UseCaseCollection</code> from wich
     *            this <code>TreeModel</code> is
     *            constructed.
     */
    public UseCaseTreeModel(UseCaseCollection u) {
        super();
        this.collection = u;
        this.root = new FilesTreeItem(this, null);
        u.addObserver(this);
    }

    public Object getRoot() {
        return this.root;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#getChild(java.lang.Object,
     *      int)
     */
    public Object getChild(Object parent, int index) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            TreeItem result = null;
            if (children != null) {
                result = children.get(index);
            }
            return result;
        } else throw new UnsupportedOperationException(
                "parent is not instance of TreeItem");
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#getChildCount(java.lang.Object)
     */
    public int getChildCount(Object parent) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            int result = 0;
            if (children != null) {
                result = children.size();
            }
            return result;
        } else throw new UnsupportedOperationException(
                "parent is not instance of TreeItem");
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#isLeaf(java.lang.Object)
     */
    public boolean isLeaf(Object node) {
        return getChildCount(node) == 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#valueForPathChanged(javax.swing.tree.TreePath,
     *      java.lang.Object)
     */
    public void valueForPathChanged(TreePath path,
            Object newValue) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#getIndexOfChild(java.lang.Object,
     *      java.lang.Object)
     */
    public int getIndexOfChild(Object parent, Object child) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            int index = -1;
            if (children != null) {
                index = children.indexOf(child);
            }
            return index;
        } else throw new UnsupportedOperationException(
                "parent is not instance of TreeItem");
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#addTreeModelListener(javax.swing.event.TreeModelListener)
     */
    public void addTreeModelListener(TreeModelListener l) {
        listeners.add(l);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeModel#removeTreeModelListener(javax.swing.event.TreeModelListener)
     */
    public void removeTreeModelListener(TreeModelListener l) {
        listeners.remove(l);
    }

    /**
     * <p>
     * Returns <code>UseCaseCollection</code> from wich
     * this <code>TreeModel</code> is contructed.
     * 
     * @return <code>UseCaseCollection</code> from wich
     *         this <code>TreeModel</code> is contructed.
     */
    public UseCaseCollection getUseCaseCollection() {
        return this.collection;
    }

    /**
     * <p>
     * Method for clearing this <code>TreeModel</code>.
     * </p>
     * 
     * <p>
     * When called clears contest of this data model and
     * also clears contest of <code>UseCaseCollection</code>
     * from wich this <code>TreeModel</code> is
     * constructed.
     * </p>
     * 
     * 
     * 
     */
    public void clear() {
        this.root = new FilesTreeItem(this, null);
        this.collection.clear();
    }
}
